@@ -40,10 +40,9 @@
 
 #include <Python.h> /* make sure Python.h is first */
 #include <stdlib.h>
+#include "ferret.h"
 
-void set_null_c_string_array_(fer_ptr, nstr)
-     char*** fer_ptr;
-     int* nstr;
+void FORTRAN(set_null_c_string_array)(char ***fer_ptr, int *nstr)
 {
    char** each_str_ptr;
    int i;
@@ -52,8 +51,8 @@ void set_null_c_string_array_(fer_ptr, nstr)
 
    for (i=0; i<*nstr; i++) {
       if ( *each_str_ptr != NULL )
-         free(*each_str_ptr);
-      *each_str_ptr = (char *) malloc(sizeof(char));
+         PyMem_Free(*each_str_ptr);
+      *each_str_ptr = (char *) PyMem_Malloc(sizeof(char));
       if ( *each_str_ptr == NULL )
          abort();
       **each_str_ptr = '\0';
