@@ -24,7 +24,6 @@
 #include "cferbind.h"
 #include "pyferret.h"
 
-
 #ifdef VERBOSEDEBUG
 #include <stdio.h>
 #include <stdlib.h>
@@ -88,6 +87,7 @@ void fgderrmsg_(char *errmsg, int *errmsglen)
  *     titlelen: actual length of the title
  *     visible: display Window on start-up?
  *     noalpha: do not use the alpha channel (opacity) in colors
+ *     rasteronly: only create raster images
  *
  * Returns a pointer to the window object created.
  * If an error occurs, NULL is returned and
@@ -95,11 +95,13 @@ void fgderrmsg_(char *errmsg, int *errmsglen)
  */
 grdelType grdelWindowCreate(const char *engine, int enginelen,
                             const char *title, int titlelen, 
-                            grdelBool visible, grdelBool noalpha)
+                            grdelBool visible, grdelBool noalpha, 
+                            grdelBool rasteronly)
 {
     GDWindow *window;
     PyObject *visiblebool;
     PyObject *noalphabool;
+    PyObject *rasteronlybool;
 
     /* Allocate memory for this GDWindow */
     window = (GDWindow *) PyMem_Malloc(sizeof(GDWindow));
@@ -118,7 +120,7 @@ grdelType grdelWindowCreate(const char *engine, int enginelen,
      * This will fail if it is a Python-based engine.
      */
     window->bindings.cferbind = cferbind_createWindow(engine, enginelen,
-                                         title, titlelen, visible, noalpha);
+                           title, titlelen, visible, noalpha, rasteronly);
     if ( window->bindings.cferbind != NULL ) {
         /* Success - engine found; done */
 #ifdef VERBOSEDEBUG
@@ -151,10 +153,16 @@ grdelType grdelWindowCreate(const char *engine, int enginelen,
         noalphabool = Py_True;
     else
         noalphabool = Py_False;
+    if ( rasteronly )
+        rasteronlybool = Py_True;
+    else
+        rasteronlybool = Py_False;
     window->bindings.pyobject = 
             PyObject_CallMethod(pyferret_graphbind_module_pyobject, 
-                                "createWindow", "s#s#OO", engine, enginelen,
-                                title, titlelen, visiblebool, noalphabool);
+                                "createWindow", "s#s#OOO", 
+                                engine, enginelen,
+                                title, titlelen, 
+                                visiblebool, noalphabool, rasteronlybool);
     if ( window->bindings.pyobject == NULL ) {
         sprintf(grdelerrmsg, "grdelWindowCreate: error when calling createWindow "
                              "in pyferret.graphbind: %s", pyefcn_get_error());
@@ -851,7 +859,11 @@ grdelBool grdelWindowSave(grdelType window, const char *filename,
                 return 0;
             }
             for (k = 0; k < numannotations; k++) {
+#if PY_MAJOR_VERSION > 2
+                annostrobj = PyUnicode_FromString((char *) annotations[k * 8 / sizeof(void *)]);
+#else
                 annostrobj = PyString_FromString((char *) annotations[k * 8 / sizeof(void *)]);
+#endif
                 if ( annostrobj == NULL ) {
                     Py_DECREF(annostuple);
                     strcpy(grdelerrmsg, "grdelWindowSave: unexpected error, "
@@ -954,6 +966,13 @@ grdelBool grdelWindowScreenInfo(grdelType window, float *dpix, float *dpiy,
         return 0;
     }
 
+#ifdef VERBOSEDEBUG
+    fprintf(debuglogfile, "grdelWindowScreenInfo response: "
+                          "dpix = %f, dpiy = %f, screenwidth = %d, screenheight = %d\n",
+                          *dpix, *dpiy, *screenwidth, *screenheight);
+    fflush(debuglogfile);
+#endif
+
     return 1;
 }
 
@@ -997,7 +1016,9 @@ int  grdelWindowSetDpi(grdelType window, float newdpi)
                             "no bindings associated with this Window");
         return 0;
     }
-    /* (at this time) anything with only C bindings has no display associated with it */
+    if ( mywindow->bindings.cferbind->setWindowDpi == NULL ) {
+        return -1;
+    }
     mywindow->bindings.cferbind->setWindowDpi(mywindow->bindings.cferbind, (double) newdpi);
     return 1;
 }
@@ -1070,17 +1091,19 @@ grdelBool grdelWindowSetWidthFactor(grdelType window, float widthfactor)
  *     titlelen: actual length of the title
  *     visible: display Window on start-up? If zero, no; if non-zero, yes.
  *     noalpha: do not use the alpha channel (opacity) in colors ?
+ *     rasteronly: only create raster images
  * Output Arguments:
  *     window: the window object created, or zero if failure.
  *             Use fgderrmsg_ to retreive the error message.
  */
 void fgdwincreate_(void **window, char *engine, int *enginelen,
-                   char *title, int *titlelen, int *visible, int *noalpha)
+                   char *title, int *titlelen, int *visible, 
+                   int *noalpha, int *rasteronly)
 {
     grdelType mywindow;
 
-    mywindow = grdelWindowCreate(engine, *enginelen, title, 
-                                 *titlelen, *visible, *noalpha);
+    mywindow = grdelWindowCreate(engine, *enginelen, title, *titlelen, 
+                                 *visible, *noalpha, *rasteronly);
     *window = mywindow;
 }
 
@@ -1276,9 +1299,8 @@ void fgdwinsetvis_(int *success, void **window, int *visible)
  *     yinches: vertical size of vector image in inches
  *     xpixels: horizontal size of raster image in pixels
  *     ypixels: vertical size of raster image in pixels
- *     memory: ferret memory containing annotation C strings; 
- *             pointers are always 8 bytes apart
- *     firststr: offset into memory of the first annotation C string
+ *     firststr: ferret memory pointer to the first annotation C string;
+ *               pointers are always 8 bytes apart
  *     numstr: number of annotation C strings
  *
  * If formatlen is zero, the fileformat is guessed from the
@@ -1291,15 +1313,14 @@ void fgdwinsetvis_(int *success, void **window, int *visible)
 void fgdwinsave_(int *success, void **window, char *filename, int *namelen,
                  char *fileformat, int *formatlen, int *transparentbkg,
                  float *xinches, float *yinches, int *xpixels, int *ypixels,
-                 void **memory, int *firststr, int *numstr)
+                 void **firststr, int *numstr)
 {
     grdelBool result;
 
     result = grdelWindowSave(*window, filename, *namelen,
                              fileformat, *formatlen, *transparentbkg,
                              *xinches, *yinches, *xpixels, *ypixels,
-                             &(memory[(*firststr) * 8 / sizeof(void *)]), 
-                             *numstr);
+                             firststr, *numstr);
     *success = result;
 }
 
