@@ -48,6 +48,12 @@
 					 definition of macro DFTYPE in ferretmacros.h.
  *acm*  5/13      - ticket 2066: For double-precision Ferret reading numeric data,
                     sscanf needs %lf instead of %f, and dummy needs to be double prec.
+  V701 7/16 *acm*   ticket 2450: add date-time and euro-date-time field types
+                    ticket 2448: for 2-digit years, put years prior to 50 into the 2000s
+                    ticket 2449: report incorrect choice of date/ eurodate as an error
+  V702 10/16 *acm*  ticket 2472: Allow yyyy/dd/mm in any of the date types.   
+  V702 11/16 *acm*  handling read errors in date/time reading
+  V702  1/17 *sh*   removal of block-oriented memory management
 */
 
 
@@ -89,23 +95,28 @@
  nfields - number of fields to decode on each record
  field_types - type of each field (input)
  mrlist - list of integers pointing to Ferret memory blocks
- memptr - pointer to base of Ferret "heap" storage
- mr_blk1 - memory chunk numbers indexed by mr_list
- mblk_size - chunk size within Ferret heap
+ mr_ptrs_val - integer containing the c pointer to array of mr memory pointers 
  mr_bad_flags - missing value flags indexed by mr_list
  *
  */
 
 
+void FORTRAN(decode_file_jacket)
+		( char* fname, char *recptr, char *delims, int *skip,
+		  int* maxrec, int* reclen, int* nfields,
+		  int field_type[], int* nrec,
+		  int mrlist[], long* mr_ptrs_val,
+		  DFTYPE mr_bad_flags[], char ***mr_c_ptr, int* status)
+/* 1/17 --- pre-dynamic memory call
 void FORTRAN(decode_file_jacket)
 		( char* fname, char *recptr, char *delims, int *skip,
 		  int* maxrec, int* reclen, int* nfields,
 		  int field_type[], int* nrec,
 		  int mrlist[], DFTYPE *memptr, int mr_blk1[], int* mblk_size,
-		  DFTYPE mr_bad_flags[], char ***mr_c_ptr)
-
+		  DFTYPE mr_bad_flags[], char ***mr_c_ptr, int* status)
+*/
 {
-
+  DFTYPE** mr_ptrs = (DFTYPE**) *mr_ptrs_val;
   DFTYPE **numeric_fields  = (DFTYPE **) malloc(sizeof(DFTYPE*) * (*nfields));
   DFTYPE *bad_flags        = (DFTYPE *)  malloc(sizeof(DFTYPE) * (*nfields));
 
@@ -126,21 +137,21 @@ void FORTRAN(decode_file_jacket)
       if (field_type[i] == FTYP_CHARACTER )
 	{
 	  /* *kob* make sure were using size of real*4 float */
-	  text_fields[i] = (char**) (memptr + ((mr_blk1[mr]-1)*(*mblk_size)*4)/sizeof(float));
+	  text_fields[i] = (char**) mr_ptrs[i];
 	  mr_c_ptr[mr*pinc] = text_fields[i];
 	}
       else if (field_type[i] != FTYP_MISSING )
 	{
-	  numeric_fields[i] = memptr + (mr_blk1[mr]-1)*(*mblk_size);
+	  numeric_fields[i] = mr_ptrs[i];
 	  mr_c_ptr[mr*pinc] = (char**) NULL;
 	}
       /*
 	isolate the bad data flags that correspond to the numeric fields
       */
-      if ( (field_type[i]!=FTYP_MISSING) && (field_type[i]!=FTYP_CHARACTER) )
-	bad_flags[i] = mr_bad_flags[mr];
-      else
+      if ( (field_type[i]==FTYP_MISSING) || (field_type[i]==FTYP_CHARACTER) )
 	bad_flags[i] = 0.0;
+      else
+	bad_flags[i] = mr_bad_flags[mr];
     }
 
   /*
@@ -149,7 +160,7 @@ void FORTRAN(decode_file_jacket)
   decode_file (fname, recptr, delims, skip, 
 	       maxrec, reclen, nfields,
 	       field_type, nrec,
-	       numeric_fields, text_fields, bad_flags);
+	       numeric_fields, text_fields, bad_flags, status);
 
   free(numeric_fields);
   free(text_fields);
@@ -185,13 +196,14 @@ void FORTRAN(decode_file_jacket)
 int decode_file (char* fname, char *recptr, char *delims, int *skip, 
 			  int* maxrec, int* reclen, int* nfields,
 			  int field_type[], int* nrec, DFTYPE** numeric_fields,
-			  char*** text_fields, DFTYPE bad_flags[])
+			  char*** text_fields, DFTYPE bad_flags[], int* status)
 
 {
 
   FILE *fp;
   int slen, i;
   int pinc = 8/sizeof(char*);  /* pointers spaced 8 bytes apart */
+  char *startptr;
 
   *nrec = 0;
 
@@ -210,16 +222,17 @@ int decode_file (char* fname, char *recptr, char *delims, int *skip,
       if ( fgets(recptr,*reclen,fp) )
 	{
 	  /* skip leading blanks */
-	  while (*recptr==' ')
-	    recptr++;
+          startptr = recptr;
+	  while (*startptr ==' ')
+	    startptr++;
 
 	  /* overwrite the newline record terminator with a NULL */
-	  if ((slen = strlen(recptr)) > 0)
-	    if (recptr[slen-1] == '\n')
-	      recptr[slen-1] = '\0';
+	  if ((slen = strlen(startptr)) > 0)
+	    if (startptr[slen-1] == '\n')
+	      startptr[slen-1] = '\0';
 	  
-	  decodeRec(recptr, delims, nfields, field_type, *nrec,
-		    numeric_fields, text_fields, bad_flags);
+	  decodeRec(startptr, delims, nfields, field_type, *nrec,
+		    numeric_fields, text_fields, bad_flags, status);
 
 #ifdef diagnostic_output	  /* ************* */
 	  for (i=0; i<(*nfields); i++)
@@ -228,7 +241,8 @@ int decode_file (char* fname, char *recptr, char *delims, int *skip,
 	    else if (field_type[i] != FTYP_MISSING)
 	      printf( "%d numeric: %f\n",i,(*(numeric_fields+i))[(*nrec)] );
 #endif                            /* ************* */
-
+	   if (*status != 3) return 0;
+	   
 	  (*nrec)++;
 	}
     }
@@ -267,6 +281,7 @@ int FORTRAN(anal_file) (char* fname, char *recptr, char *delims, int* skip,
   FILE *fp;
   int slen, i, rec;
   int nsuccess = 0;
+  char *startptr;
 
   fp = fopen(fname,"r");
 
@@ -290,14 +305,15 @@ int FORTRAN(anal_file) (char* fname, char *recptr, char *delims, int* skip,
 	  rec++;
 
 	  /* skip leading blanks */
-	  while (*recptr==' ')
-	    recptr++;
+          startptr = recptr;
+	  while (*startptr ==' ')
+	    startptr++;
 
 	  /* overwrite the newline record terminator with a NULL */
-	  if ((slen = strlen(recptr)) > 0)
-	    recptr[slen-1] = '\0';
+	  if ((slen = strlen(startptr)) > 0)
+	    startptr[slen-1] = '\0';
 	  
-	  analRec(recptr, delims, nfields, field_type, *max_fields);
+	  analRec(startptr, delims, nfields, field_type, *max_fields);
 
 	  /* check for unknown field types */
 	  i = 0;
@@ -352,21 +368,31 @@ int FORTRAN(anal_file) (char* fname, char *recptr, char *delims, int* skip,
 
 int decodeRec(char *recptr, char *delims, int* nfields, int field_type[],
 	      int rec,
-	      DFTYPE** numeric_fields, char*** text_fields, DFTYPE bad_flags[])
+	      DFTYPE** numeric_fields, char*** text_fields, DFTYPE bad_flags[],
+	      int* status)
 {
 
   char *p, *pnext, str1[2], errstr[2];
   double dummy;
   DFTYPE rdum;
 
-  int idummy1, idummy2, idummy3, i;
+  int idummy1, idummy2, idummy3, idummy4, idummy5, i;
   char blankstr[] = " ";
   double days_1900 = 59958230400.0 / (60.*60.*24.); 
 /*  int days_1900 = 693961;  */
 
   int pinc = 8/sizeof(char*);  /* pointers spacd 8 bytes apart */
   int slen;     /* kob 12/01 needed to check for numberical string ending in e/E */
+  int ndum;
+  int break_century;
+  int rstatus;
+  double tpart;
   p = recptr;
+  rstatus = 3; 
+
+/* 2-digit years need to be assigned to a century. 
+   will break after 2049 or before 1950  (ACM 7/2016 was year 20 */
+  break_century = 50; 
 
   for (i=0; i<*nfields; i++) {
     pnext = nexstrtok(p, delims);
@@ -384,6 +410,7 @@ int decodeRec(char *recptr, char *delims, int* nfields, int field_type[],
       }
 
     } else {
+	  *status = 3;
       switch (field_type[i]) {
 	
 	/* latitude */
@@ -419,74 +446,273 @@ int decodeRec(char *recptr, char *delims, int* nfields, int field_type[],
 	
 	/* date */
       case FTYP_DATE:
-	if (sscanf(p,"%d/%d/%d%1s",
-		   &idummy1,&idummy2,&idummy3,errstr) == 3) {
-	   
-      /*printf("%d %d %d \n", idummy1,idummy2,idummy3); */
-	  /* need to check for 4 digit year - mm/dd/yyyy *kob* */
-	  if (idummy3 < 100) {
-	    if (idummy3 < 20)   /* will break after 2019 or before 1920 */
+	(*(numeric_fields+i))[rec] = bad_flags[i];
+	if (sscanf(p,"%d/%d/%d%1s",&idummy1,&idummy2,&idummy3,errstr) == 3) {
+	  /* need to check to see if idummy3 which contains the year is in 
+	     the form YY or YYYY     *kob*  10/02  */
+		 
+	  /* Date of 00/00/00 is bad-data */
+	  if (idummy1 == 0 || idummy2 == 0 || idummy3 == 0) {
+		  (*(numeric_fields+i))[rec] = bad_flags[i];
+		  break;
+	  }
+
+	  /* check for yyyy/mm/dd */
+	  if (idummy1 > 1800) {
+		  (*(numeric_fields+i))[rec] =
+		  days_from_day0_(&days_1900,&idummy1,&idummy2,&idummy3,&rdum,status);
+		  (*(numeric_fields+i))[rec] = rdum;
+		  if (*status != 3) (*(numeric_fields+i))[rec] = bad_flags[i];
+		  break;
+	  }
+
+	  /* Check for 2-digit or 4-digit year */
+	  if (idummy3 < 100) { 
+	    if (idummy3 < break_century)   /* assign 2-digit year to century*/
 	      idummy3 += 2000;
 	    else
 	      idummy3 += 1900;
 	  }
-	  (*(numeric_fields+i))[rec] =
-	    days_from_day0_(&days_1900,&idummy3,&idummy1,&idummy2,&rdum);
-	  (*(numeric_fields+i))[rec] = rdum;
 
-	  /* force dates with dashes "-" to be in yyyy-mm-dd format *kob* */
+	  (*(numeric_fields+i))[rec] =
+	    days_from_day0_(&days_1900,&idummy3,&idummy1,&idummy2,&rdum,status);
+	  (*(numeric_fields+i))[rec] = rdum;
+	  if (*status != 3) (*(numeric_fields+i))[rec] = bad_flags[i];
+	/* force dates with dashes "-" to be in yyyy-mm-dd format *kob* */
 	} else if (sscanf(p,"%4d-%2d-%2d%1s",
 			  &idummy1,&idummy2,&idummy3,errstr) == 3) {
 	  (*(numeric_fields+i))[rec] =
-	    days_from_day0_(&days_1900,&idummy1,&idummy2,&idummy3,&rdum);
-	  (*(numeric_fields+i))[rec] = rdum;
+	    days_from_day0_(&days_1900,&idummy1,&idummy2,&idummy3,&rdum,status);
+	  (*(numeric_fields+i))[rec] = rdum; 
+	  if (*status != 3) (*(numeric_fields+i))[rec] = bad_flags[i];
 	  }
+	/* yyyyddmm date */
 	else if ( (sscanf(p,"%4d%2d%2d%1s",&idummy1,&idummy2,&idummy3,str1)==3)
 	      && idummy1>0
 	      && idummy2>=1 && idummy2<=12
 	      && idummy3>=1 && idummy3<=31 ) {
 	  (*(numeric_fields+i))[rec] =
-	    days_from_day0_(&days_1900,&idummy1,&idummy2,&idummy3,&rdum);
-	  (*(numeric_fields+i))[rec] = rdum;
+	    days_from_day0_(&days_1900,&idummy1,&idummy2,&idummy3,&rdum,status);
+	  (*(numeric_fields+i))[rec] = rdum; }
+	else
+	  (*(numeric_fields+i))[rec] = bad_flags[i];
+	break;
+
+	/* Date and time
+	   Parses fields that are date time separated by a space.
+	   It would be good to also allow for Z or T separator
+	   Currently those can be handled as separate date and time
+	   fields with T or Z given as a delimiter. */
+
+	      case FTYP_DATIME:
+			  
+	if (sscanf(p,"%d/%d/%d %d:%d:%lf%1s",&idummy1,&idummy2,&idummy3,&idummy4,&idummy5,&dummy,errstr) >= 4) {
+
+	  /* Date of 00/00/00 in any order is bad-data */
+	  if (idummy1 == 0 || idummy2 == 0 || idummy3 == 0) {
+		  (*(numeric_fields+i))[rec] = bad_flags[i];
+		  break;
+	  }
+
+	  ndum = sscanf(p,"%d/%d/%d %d:%d:%lf%1s",&idummy1,&idummy2,&idummy3,&idummy4,&idummy5,&dummy,errstr);
+	  tpart = -999;
+	  if (ndum == 6) tpart = idummy4 + idummy5/60. + dummy/3600.;
+	  if (ndum == 5) tpart = idummy4 + idummy5/60.;	
+
+	  /* check for yyyy/mm/dd */
+	  if (idummy1 > 1800) {
+		  (*(numeric_fields+i))[rec] =
+		  days_from_day0_(&days_1900,&idummy1,&idummy2,&idummy3,&rdum,status);
+		  if (*status != 3) (*(numeric_fields+i))[rec] = bad_flags[i];
+	  } else {
+
+	  /* Check for 2-digit or 4-digit year */
+		  if (idummy3 < 100) { 
+			  if (idummy3 < break_century)   /* assign 2-digit year to century*/
+			  idummy3 += 2000;
+		  else
+			  idummy3 += 1900;
+		  }
+		  
+		  (*(numeric_fields+i))[rec] =
+		  days_from_day0_(&days_1900,&idummy3,&idummy1,&idummy2,&rdum,status);
+		  if (*status != 3) (*(numeric_fields+i))[rec] = bad_flags[i];
+	  }
+
+	  (*(numeric_fields+i))[rec] = rdum + tpart/24.;
+	  if (tpart == -999) (*(numeric_fields+i))[rec] = bad_flags[i];
+
+	  /* force dates with dashes "-" to be in yyyy-mm-dd format *kob* */
+	} else if (sscanf(p,"%4d-%2d-%2d %d:%d:%lf%1s",
+			  &idummy1,&idummy2,&idummy3,&idummy4,&idummy5,&dummy,errstr)  >= 4) {
+		
+		ndum = sscanf(p,"%4d-%2d-%2d %d:%d:%lf%1s",&idummy1,&idummy2,&idummy3,&idummy4,&idummy5,&dummy,errstr);
+		tpart = -999;	
+		if (ndum == 6) tpart = idummy4 + idummy5/60. + dummy/3600.;
+		if (ndum == 5) tpart = idummy4 + idummy5/60.;
+
+	  (*(numeric_fields+i))[rec] =
+	    days_from_day0_(&days_1900,&idummy1,&idummy2,&idummy3,&rdum,status);
+	  (*(numeric_fields+i))[rec] = rdum + tpart/24.; 
+	  if (tpart == -999) (*(numeric_fields+i))[rec] = bad_flags[i];
+	  if (*status != 3) (*(numeric_fields+i))[rec] = bad_flags[i];
+	  }
+
+	/* check for yyyymmdd date */
+	else if ( (sscanf(p,"%4d%2d%2d%1s",&idummy1,&idummy2,&idummy3,&idummy4,&idummy5,&dummy,str1) >= 4)
+	      && idummy1>0
+	      && idummy3>=1 && idummy3<=12
+	      && idummy2>=1 && idummy2<=31 ) {
+		ndum = sscanf(p,"%4d%2d%2d%1s",&idummy1,&idummy2,&idummy3,&idummy4,&idummy5,&dummy,str1);
+		tpart = -999;	
+		if (ndum == 6) tpart = idummy4 + idummy5/60. + dummy/3600.;
+		if (ndum == 5) tpart = idummy4 + idummy5/60.;	
+
+		(*(numeric_fields+i))[rec] =
+		days_from_day0_(&days_1900,&idummy1,&idummy2,&idummy3,&rdum,status);
+		(*(numeric_fields+i))[rec] = rdum + tpart/24.; 	
+	    if (tpart == -999) (*(numeric_fields+i))[rec] = bad_flags[i];
+		if (*status != 3) (*(numeric_fields+i))[rec] = bad_flags[i];
 	  }
 	else
 	  (*(numeric_fields+i))[rec] = bad_flags[i];
 	break;
 	
-	/* date */
+	/* Euro-date */
       case FTYP_EURODATE:
 	if (sscanf(p,"%d/%d/%d%1s",&idummy1,&idummy2,&idummy3,errstr) == 3) {
 	  /* need to check to see if idummy3 which contains the year is in 
 	     the form YY or YYYY     *kob*  10/02  */
+		 
+	  /* Date of 00/00/00 is bad-data */
+	  if (idummy1 == 0 || idummy2 == 0 || idummy3 == 0) {
+		  (*(numeric_fields+i))[rec] = bad_flags[i];
+		  break;
+	  }
+
+	  /* check for yyyy/mm/dd */
+	  if (idummy1 > 1800) {
+		  (*(numeric_fields+i))[rec] =
+		  days_from_day0_(&days_1900,&idummy1,&idummy2,&idummy3,&rdum,status);
+		  (*(numeric_fields+i))[rec] = rdum;
+		  if (*status != 3) (*(numeric_fields+i))[rec] = bad_flags[i];
+		  break;
+	  }
+
+	  /* Check for 2-digit or 4-digit year */
 	  if (idummy3 < 100) { 
-	    if (idummy3 < 20)   /* will break after 2019 or before 1920 */
+	    if (idummy3 < break_century)   /* assign 2-digit year to century*/
 	      idummy3 += 2000;
 	    else
 	      idummy3 += 1900;
 	  }
+
 	  (*(numeric_fields+i))[rec] =
-	    days_from_day0_(&days_1900,&idummy3,&idummy2,&idummy1,&rdum);
+	    days_from_day0_(&days_1900,&idummy3,&idummy2,&idummy1,&rdum,status);
 	  (*(numeric_fields+i))[rec] = rdum;
+	  if (*status != 3) (*(numeric_fields+i))[rec] = bad_flags[i];
 	  /* force dates with dashes "-" to be in yyyy-mm-dd format *kob* */
 	} else if (sscanf(p,"%4d-%2d-%2d%1s",
 			  &idummy1,&idummy2,&idummy3,errstr) == 3) {
 	  (*(numeric_fields+i))[rec] =
-	    days_from_day0_(&days_1900,&idummy1,&idummy2,&idummy3,&rdum);
-	  (*(numeric_fields+i))[rec] = rdum; }
+	    days_from_day0_(&days_1900,&idummy1,&idummy2,&idummy3,&rdum,status);
+	  (*(numeric_fields+i))[rec] = rdum;
+	  if (*status != 3) (*(numeric_fields+i))[rec] = bad_flags[i]; }
 	/* add check for yyyyddmm euro date *kob* */
 	else if ( (sscanf(p,"%4d%2d%2d%1s",&idummy1,&idummy2,&idummy3,str1)==3)
 	      && idummy1>0
 	      && idummy3>=1 && idummy3<=12
 	      && idummy2>=1 && idummy2<=31 ) {
 	  (*(numeric_fields+i))[rec] =
-	    days_from_day0_(&days_1900,&idummy1,&idummy3,&idummy2,&rdum);
-	  (*(numeric_fields+i))[rec] = rdum; }
+	    days_from_day0_(&days_1900,&idummy1,&idummy3,&idummy2,&rdum,status);
+	  (*(numeric_fields+i))[rec] = rdum;
+	  if (*status != 3) (*(numeric_fields+i))[rec] = bad_flags[i]; }
+	else
+	  (*(numeric_fields+i))[rec] = bad_flags[i];
+	break;
+
+	/* Euro-date and time
+	   Parses fields that are date time separated by a space.
+	   It would be good to also allow for Z or T separator
+	   Currently those can be handled as separate date and time
+	   fields with T or Z given as a delimiter. */
+
+	      case FTYP_EDATIME:
+
+	if (sscanf(p,"%d/%d/%d %d:%d:%lf%1s",&idummy1,&idummy2,&idummy3,&idummy4,&idummy5,&dummy,errstr) >= 4) {
+
+	  /* Date of 00/00/00 in any order is bad-data */
+	  if (idummy1 == 0 || idummy2 == 0 || idummy3 == 0) {
+		  (*(numeric_fields+i))[rec] = bad_flags[i];
+		  break;
+	  }
+
+	  ndum = sscanf(p,"%d/%d/%d %d:%d:%lf%1s",&idummy1,&idummy2,&idummy3,&idummy4,&idummy5,&dummy,errstr);
+	  tpart = -999;
+	  if (ndum == 6) tpart = idummy4 + idummy5/60. + dummy/3600.;
+	  if (ndum == 5) tpart = idummy4 + idummy5/60.;	
+
+	  /* check for yyyy/mm/dd */
+	  if (idummy1 > 1800) {
+		  (*(numeric_fields+i))[rec] =
+		  days_from_day0_(&days_1900,&idummy1,&idummy2,&idummy3,&rdum,status);
+
+	  } else {
+
+	  /* Check for 2-digit or 4-digit year */
+		  if (idummy3 < 100) { 
+			  if (idummy3 < break_century)   /* assign 2-digit year to century*/
+			  idummy3 += 2000;
+		  else
+			  idummy3 += 1900;
+		  }
+		  
+		  (*(numeric_fields+i))[rec] =
+		  days_from_day0_(&days_1900,&idummy3,&idummy2,&idummy1,&rdum,status);
+	  }
+
+	  (*(numeric_fields+i))[rec] = rdum + tpart/24.;
+	  if (*status != 3) (*(numeric_fields+i))[rec] = bad_flags[i];
+	  if (tpart == -999) (*(numeric_fields+i))[rec] = bad_flags[i];
+
+	  /* force dates with dashes "-" to be in yyyy-mm-dd format *kob* */
+	} else if (sscanf(p,"%4d-%2d-%2d %d:%d:%lf%1s",
+			  &idummy1,&idummy2,&idummy3,&idummy4,&idummy5,&dummy,errstr)  >= 4) {
+		
+		ndum = sscanf(p,"%4d-%2d-%2d %d:%d:%lf%1s",&idummy1,&idummy2,&idummy3,&idummy4,&idummy5,&dummy,errstr);
+		tpart = -999;	
+		if (ndum == 6) tpart = idummy4 + idummy5/60. + dummy/3600.;
+		if (ndum == 5) tpart = idummy4 + idummy5/60.;
+
+	  (*(numeric_fields+i))[rec] =
+	    days_from_day0_(&days_1900,&idummy1,&idummy2,&idummy3,&rdum,status);
+	  (*(numeric_fields+i))[rec] = rdum + tpart/24.; 
+	  if (*status != 3) (*(numeric_fields+i))[rec] = bad_flags[i];
+	  if (tpart == -999) (*(numeric_fields+i))[rec] = bad_flags[i];
+	  }
+
+	/* add check for yyyyddmm euro date *kob* */
+	else if ( (sscanf(p,"%4d%2d%2d%1s",&idummy1,&idummy2,&idummy3,&idummy4,&idummy5,&dummy,str1) >= 4)
+	      && idummy1>0
+	      && idummy3>=1 && idummy3<=12
+	      && idummy2>=1 && idummy2<=31 ) {
+		ndum = sscanf(p,"%4d%2d%2d%1s",&idummy1,&idummy2,&idummy3,&idummy4,&idummy5,&dummy,str1);
+		tpart = -999;	
+		if (ndum == 6) tpart = idummy4 + idummy5/60. + dummy/3600.;
+		if (ndum == 5) tpart = idummy4 + idummy5/60.;	
+
+		(*(numeric_fields+i))[rec] =
+		days_from_day0_(&days_1900,&idummy1,&idummy3,&idummy2,&rdum,status);
+		(*(numeric_fields+i))[rec] = rdum + tpart/24.;
+	    if (tpart == -999) (*(numeric_fields+i))[rec] = bad_flags[i];
+	    if (*status != 3) (*(numeric_fields+i))[rec] = bad_flags[i];  }
 	else
 	  (*(numeric_fields+i))[rec] = bad_flags[i];
 	break;
 	
 	/* time */
       case FTYP_TIME:
+	(*(numeric_fields+i))[rec] = bad_flags[i];
 	if (sscanf(p,"%d:%d:%lf%1s",&idummy1,&idummy2,&dummy,errstr) == 3)
 	  (*(numeric_fields+i))[rec] = idummy1 + idummy2/60. + dummy/3600.;
 	else if (sscanf(p,"%d:%d%1s",&idummy1,&idummy2,errstr) == 2)
@@ -529,8 +755,15 @@ int decodeRec(char *recptr, char *delims, int* nfields, int field_type[],
       }
     }
 
-    p = pnext;
+/* if bad status flag out of a date conversion, make the date the bad-flag
+   Then the user can deal with undefined times as desired */
+
+ 	if (*status != 3 && rstatus == 3) rstatus = -1* (i+1);
+
+    p = pnext;	
     }
+
+   if (rstatus != 3) *status = rstatus; 
 }
 
 /*
