@@ -7,8 +7,29 @@ Project (TMAP) of the National Oceanographic and Atmospheric
 Administration's (NOAA) Pacific Marine Environmental Lab (PMEL).
 '''
 
-from PyQt4.QtCore import Qt, QPointF, QSizeF
-from PyQt4.QtGui import QBrush, QColor, QFont, QPainterPath, QPen
+import sys
+
+if sys.version_info[0] > 2:
+    # First try to import PyQt5, then try PyQt4 if that fails
+    try:
+        import PyQt5
+        QT_VERSION = 5
+    except ImportError:
+        import PyQt4
+        QT_VERSION = 4
+else:
+    # PyQt5 requires Python3.x, so only try PyQt4
+    import PyQt4
+    QT_VERSION = 4
+
+# Now that the PyQt version is determined, import the parts
+# allowing any import errors to propagate out
+if QT_VERSION == 5:
+    from PyQt5.QtCore import Qt, QPointF, QSizeF
+    from PyQt5.QtGui  import QBrush, QColor, QFont, QPainterPath, QPen
+else:
+    from PyQt4.QtCore import Qt, QPointF, QSizeF
+    from PyQt4.QtGui  import QBrush, QColor, QFont, QPainterPath, QPen
 
 
 class SidesRectF(object):
@@ -209,8 +230,7 @@ class CmndHelperPQ(object):
             elif mystyle == "diagcross":
                 mystyle = Qt.DiagCrossPattern
             else:
-                raise ValueError( self.__viewer.tr( \
-                      "Unknown brush style %1").arg(str(mystyle)) )
+                raise ValueError("Unknown brush style '%s'" % str(mystyle))
             mybrush.setStyle(mystyle)
         except KeyError:
             pass
@@ -258,8 +278,7 @@ class CmndHelperPQ(object):
             elif mystyle == "dashdotdot":
                 mystyle = Qt.DashDotDotLine
             else:
-                raise ValueError( self.__viewer.tr( \
-                      "Unknown pen style %1").arg(str(mystyle)) )
+                raise ValueError("Unknown pen style '%s'" % str(mystyle))
             mypen.setStyle(mystyle)
         except KeyError:
             pass
@@ -272,8 +291,7 @@ class CmndHelperPQ(object):
             elif mystyle == "round":
                 mystyle = Qt.RoundCap
             else:
-                raise ValueError( self.__viewer.tr( \
-                      "Unknown pen cap style %1").arg(str(mystyle)) )
+                raise ValueError("Unknown pen cap style '%s'" % str(mystyle))
             mypen.setCapStyle(mystyle)
         except KeyError:
             pass
@@ -286,8 +304,7 @@ class CmndHelperPQ(object):
             elif mystyle == "round":
                 mystyle = Qt.RoundJoin
             else:
-                raise ValueError( self.__viewer.tr( \
-                      "Unknown pen join style %1").arg(str(mystyle)) )
+                raise ValueError("Unknown pen join style '%s'" % str(mystyle))
             mypen.setJoinStyle(mystyle)
         except KeyError:
             pass
