@@ -13,25 +13,52 @@ Project (TMAP) of the National Oceanographic and Atmospheric
 Administration's (NOAA) Pacific Marine Environmental Lab (PMEL).
 '''
 
+from __future__ import print_function
+
+import sys
+import os
+import time
+import signal
+
 import sip
 try:
     sip.setapi('QVariant', 2)
 except AttributeError:
     pass
 
-from PyQt4.QtCore import Qt, QPointF, QRectF, QSize, QString, QTimer
-from PyQt4.QtGui  import QAction, QApplication, QBrush, QColor, QDialog, \
-                         QFileDialog, QImage, QLabel, QMainWindow, \
-                         QMessageBox, QPainter, QPalette, QPen, QPixmap, \
-                         QPolygonF, QPushButton, QScrollArea
+if sys.version_info[0] > 2:
+    # First try to import PyQt5, then try PyQt4 if that fails
+    try:
+        import PyQt5
+        QT_VERSION = 5
+    except ImportError:
+        import PyQt4
+        QT_VERSION = 4
+else:
+    # PyQt5 requires Python3.x, so only try PyQt4
+    import PyQt4
+    QT_VERSION = 4
+
+# Now that the PyQt version is determined, import the parts
+# allowing any import errors to propagate out
+if QT_VERSION == 5:
+    from PyQt5.QtCore    import Qt, QPointF, QRectF, QSize, QTimer
+    from PyQt5.QtGui     import QBrush, QColor, QImage, QPainter, \
+                                QPalette, QPen, QPixmap, QPolygonF
+    from PyQt5.QtWidgets import QAction, QApplication, QDialog, \
+                                QFileDialog, QLabel, QMainWindow, \
+                                QMessageBox, QPushButton, QScrollArea
+else:
+    from PyQt4.QtCore import Qt, QPointF, QRectF, QSize, QTimer
+    from PyQt4.QtGui  import QAction, QApplication, QBrush, QColor, QDialog, \
+                             QFileDialog, QImage, QLabel, QMainWindow, \
+                             QMessageBox, QPainter, QPalette, QPen, QPixmap, \
+                             QPolygonF, QPushButton, QScrollArea
 
-from cmndhelperpq import CmndHelperPQ
-from scaledialogpq import ScaleDialogPQ
 from multiprocessing import Pipe, Process
-import sys
-import time
-import os
-import signal
+
+from pipedviewer.cmndhelperpq import CmndHelperPQ
+from pipedviewer.scaledialogpq import ScaleDialogPQ
 
 
 class PipedImagerPQ(QMainWindow):
@@ -44,9 +71,7 @@ class PipedImagerPQ(QMainWindow):
           "filename":"ferret.png",
           "fileformat":"png" }
 
-    The command { "action":"exit" } will shutdown the viewer and is
-    the only way the viewer can be closed.  GUI actions can only hide
-    the viewer.
+    The command { "action":"exit" } will shutdown the viewer.
     '''
 
     def __init__(self, cmndpipe, rspdpipe):
@@ -119,9 +144,6 @@ class PipedImagerPQ(QMainWindow):
         self.__aboutqtact = QAction(self.tr("About &Qt"), self,
                                 statusTip=self.tr("Show information about the Qt library"),
                                 triggered=self.aboutQtMsg)
-        self.__exitact = QAction(self.tr("&Exit"), self,
-                                statusTip=self.tr("Shut down the viewer"),
-                                triggered=self.exitViewer)
         self.createMenus()
         # set the initial size of the viewer
         self.__framedelta = 4
@@ -146,13 +168,9 @@ class PipedImagerPQ(QMainWindow):
         sceneMenu.addAction(self.__scaleact)
         sceneMenu.addAction(self.__saveact)
         sceneMenu.addAction(self.__redrawact)
-        # sceneMenu.addSeparator()
-        # sceneMenu.addAction(self.__hideact)
         helpMenu = menuBar.addMenu(menuBar.tr("&Help"))
         helpMenu.addAction(self.__aboutact)
         helpMenu.addAction(self.__aboutqtact)
-        helpMenu.addSeparator()
-        helpMenu.addAction(self.__exitact)
 
     def resizeEvent(self, event):
         '''
@@ -171,21 +189,22 @@ class PipedImagerPQ(QMainWindow):
 
     def closeEvent(self, event):
         '''
-        Override so the viewer cannot be closed from the
-        user selecting the windowframe close ('X') button.
-        Instead only hide the window.
+        Override so will just minimize if not shutting down
         '''
-        if self.__shuttingdown:
+        # If shutting down or minimized (escape hatch), 
+        # go ahead and close/exit
+        if self.__shuttingdown or self.isMinimized():
+            self.__timer.stop()
             event.accept()
-        else:
-            event.ignore()
-            self.hide()
+            return
+        # Otherwise just minimize the window
+        event.ignore()
+        self.showMinimized()
 
     def exitViewer(self):
         '''
         Close and exit the viewer.
         '''
-        self.__timer.stop()
         self.__shuttingdown = True
         self.close()
 
@@ -200,11 +219,12 @@ class PipedImagerPQ(QMainWindow):
             "The controlling application, however, may be unaware " \
             "of these modifications made to the image. " \
             "\n\n" \
-            "Normally, the controlling program will exit the viewer " \
-            "when it is no longer needed.  The Help -> Exit menu item " \
-            "should not normally be used.  It is provided when problems " \
-            "occur and the controlling program cannot shut down the " \
-            "viewer properly. " \
+            "'Closing' this window (the window frame 'X' button) " \
+            "actually only minimizes this viewer as PyFerret expects " \
+            "it to exist until PyFerret closes it.  Selecting the " \
+            "close menu option from the *minimized* window (using " \
+            "a right mouse click) will actually close (exit) the " \
+            "viewer if PyFerret exited and left it up.  " \
             "\n\n" \
             "PipedImagerPQ was developed by the Thermal Modeling and Analysis " \
             "Project (TMAP) of the National Oceanographic and Atmospheric " \
@@ -383,8 +403,8 @@ class PipedImagerPQ(QMainWindow):
             QApplication.restoreOverrideCursor()
             raise ValueError("invalid length of an image data block")
         # update the status message to show progress
-        self.statusBar().showMessage( self.tr("Loading new image (block %1 of %2)") \
-                                          .arg(str(myblocknum)).arg(str(mynumblocks)) )
+        self.statusBar().showMessage( self.tr("Loading new image (block %s of %s)" % \
+                                              (str(myblocknum),str(mynumblocks))) )
         # assign the data
         self.__scenedata[mystartindex:myendindex] = myblockdata
         # if this is the last block of data, create and display the scene image
@@ -430,8 +450,8 @@ class PipedImagerPQ(QMainWindow):
         the window size is changed by the user and auto-scaling is turn on.
 
         Returns:
-            True if the scene was resized
-            False if the a new resize command was issued
+            True if scaling of this scene is done (no window resize)
+            False if the a window resize command was issued
         '''
         barheights = self.menuBar().height() + self.statusBar().height()
 
@@ -470,45 +490,59 @@ class PipedImagerPQ(QMainWindow):
         already scaled, that scaling is "removed" before this scaling
         factor is applied.  If resizewin is True, the main window is 
         resized to accommodate this new scaled scene size.
+
+        If factor is zero, just switch to auto-scaling at the current
+        window size.  If factor is negative, rescale using the absolute
+        value (possibly resizing the window) then switch to auto-scaling.
         '''
-        newfactor = float(factor)
-        newlabwidth = int(newfactor * self.__scenewidth + 0.5)
-        newlabheight = int(newfactor * self.__sceneheight + 0.5)
-        if (newlabwidth < self.__minsize) or (newlabheight < self.__minsize):
-            # Set to minimum size
-            if self.__scenewidth <= self.__sceneheight:
-                newfactor = float(self.__minsize) / float(self.__scenewidth)
-            else:
-                newfactor = float(self.__minsize) / float(self.__sceneheight)
+        fltfactor = float(factor)
+        if fltfactor != 0.0:
+            if resizewin:
+                # from command - turn off autoscaling for the following
+                # then turn back on if appropriate
+                self.__autoscale = False
+            newfactor = abs(fltfactor)
             newlabwidth = int(newfactor * self.__scenewidth + 0.5)
             newlabheight = int(newfactor * self.__sceneheight + 0.5)
-        oldlabwidth = int(self.__scalefactor * self.__scenewidth + 0.5)
-        oldlabheight = int(self.__scalefactor * self.__sceneheight + 0.5)
-        if (newlabwidth != oldlabwidth) or (newlabheight != oldlabheight):
-            # Set the new scaling factor
-            self.__scalefactor = newfactor
-            # Update the scene label using the current clearing color and image
-            QApplication.setOverrideCursor(Qt.WaitCursor)
-            self.statusBar().showMessage( self.tr("Scaling image") )
-            try:
-                self.updateScene()
-            finally:
-                self.statusBar().clearMessage()
-                QApplication.restoreOverrideCursor()
-        if resizewin:
-            # resize the main window (if possible)
-            barheights = self.menuBar().height() + self.statusBar().height()
-            mwheight = newlabheight + barheights + self.__framedelta
-            mwwidth = newlabwidth + self.__framedelta
-            # Do not exceed the available real estate on the screen.
-            # If autoscaling is in effect, the resize will trigger 
-            # any required adjustments.
-            scrnrect = QApplication.desktop().availableGeometry()
-            if mwwidth > 0.95 * scrnrect.width():
-                mwwidth = int(0.9 * scrnrect.width() + 0.5)
-            if mwheight > 0.95 * scrnrect.height():
-                mwheight = int(0.9 * scrnrect.height() + 0.5)
-            self.resize(mwwidth, mwheight)
+            if (newlabwidth < self.__minsize) or (newlabheight < self.__minsize):
+                # Set to minimum size
+                if self.__scenewidth <= self.__sceneheight:
+                    newfactor = float(self.__minsize) / float(self.__scenewidth)
+                else:
+                    newfactor = float(self.__minsize) / float(self.__sceneheight)
+                newlabwidth = int(newfactor * self.__scenewidth + 0.5)
+                newlabheight = int(newfactor * self.__sceneheight + 0.5)
+            oldlabwidth = int(self.__scalefactor * self.__scenewidth + 0.5)
+            oldlabheight = int(self.__scalefactor * self.__sceneheight + 0.5)
+            if (newlabwidth != oldlabwidth) or (newlabheight != oldlabheight):
+                # Set the new scaling factor
+                self.__scalefactor = newfactor
+                # Update the scene label using the current clearing color and image
+                QApplication.setOverrideCursor(Qt.WaitCursor)
+                self.statusBar().showMessage( self.tr("Scaling image") )
+                try:
+                    self.updateScene()
+                finally:
+                    self.statusBar().clearMessage()
+                    QApplication.restoreOverrideCursor()
+            if resizewin:
+                # resize the main window (if possible)
+                barheights = self.menuBar().height() + self.statusBar().height()
+                mwheight = newlabheight + barheights + self.__framedelta
+                mwwidth = newlabwidth + self.__framedelta
+                # Do not exceed the available real estate on the screen.
+                # If autoscaling is in effect, the resize will trigger 
+                # any required adjustments.
+                scrnrect = QApplication.desktop().availableGeometry()
+                if mwwidth > 0.95 * scrnrect.width():
+                    mwwidth = int(0.9 * scrnrect.width() + 0.5)
+                if mwheight > 0.95 * scrnrect.height():
+                    mwheight = int(0.9 * scrnrect.height() + 0.5)
+                self.resize(mwwidth, mwheight)
+        if fltfactor <= 0.0:
+            # From command - turn on autoscaling
+            self.__autoscale = True
+            self.autoScaleScene();
 
     def inquireSaveFilename(self):
         '''
@@ -529,30 +563,40 @@ class PipedImagerPQ(QMainWindow):
                           self.tr("XPM - X11 Pixmap (*.xpm)") ),
                         ( "xbm",
                           self.tr("XBM - X11 Bitmap (*.xbm)") ), ]
-        # tr returns QStrings so the following does not work
-        # filters = ";;".join( [ t[1] for t in formattypes ] )
-        filters = QString(formattypes[0][1])
-        for typePair in formattypes[1:]:
-            filters.append(";;")
-            filters.append(typePair[1])
-        # getSaveFileNameAndFilter does not want to accept a default filter
-        # for (fmt, fmtQName) in formattypes:
-        #     if self.__lastformat == fmt:
-        #         dfltfilter = fmtQName
-        #         break
-        # else:
-        #     dfltfilter = formattypes[0][1]
-        # getSaveFileNameAndFilter is a PyQt (but not Qt?) method
-        (fileName, fileFilter) = QFileDialog.getSaveFileNameAndFilter(self,
-                                      self.tr("Save the current image as "),
-                                      self.__lastfilename, filters)
-        if fileName:
+        if QT_VERSION == 5:
+            # tr returns Python strings in PyQt5/Python3
+            filters = ";;".join( [ t[1] for t in formattypes ] )
             for (fmt, fmtQName) in formattypes:
-                if fmtQName.compare(fileFilter) == 0:
-                    fileFormat = fmt
+                if self.__lastformat == fmt:
+                    dfltfilter = fmtQName
                     break
             else:
-                raise RuntimeError("Unexpected file format name '%s'" % fileFilter)
+                dfltfilter = formattypes[0][1]
+            (fileName, fileFilter) = QFileDialog.getSaveFileName(self,
+                self.tr("Save the current image as "), self.__lastfilename, filters, dfltfilter)
+            if fileName:
+                for (fmt, fmtQName) in formattypes:
+                    if fmtQName == fileFilter:
+                        fileFormat = fmt
+                        break
+                else:
+                    raise RuntimeError("Unexpected file format name '%s'" % fileFilter)
+        else:
+            # tr returns QStrings in PyQt4
+            filters = formattypes[0][1]
+            for typePair in formattypes[1:]:
+                filters.append(";;")
+                filters.append(typePair[1])
+            (fileName, fileFilter) = QFileDialog.getSaveFileNameAndFilter(self,
+                 self.tr("Save the current image as "), self.__lastfilename, filters)
+            if fileName:
+                for (fmt, fmtQName) in formattypes:
+                    if fmtQName.compare(fileFilter) == 0:
+                        fileFormat = fmt
+                        break
+                else:
+                    raise RuntimeError("Unexpected file format name '%s'" % fileFilter)
+        if fileName:
             self.saveSceneToFile(fileName, fileFormat, None, None)
             self.__lastfilename = fileName
             self.__lastformat = fileFormat
@@ -621,7 +665,8 @@ class PipedImagerPQ(QMainWindow):
             mypainter.drawImage(trgrect, self.__sceneimage, srcrect, Qt.AutoColor)
             mypainter.end()
             # save the image to file
-            myimage.save(myfilename, myformat)
+            if not myimage.save(myfilename, myformat):
+                raise ValueError("Unable to save the plot as " + myfilename)
         finally:
             self.statusBar().clearMessage()
             QApplication.restoreOverrideCursor()
@@ -651,10 +696,13 @@ class PipedImagerPQ(QMainWindow):
             # EOFError should never arise from recv since
             # the call is after poll returns True
             (exctype, excval) = sys.exc_info()[:2]
-            if excval:
-                self.__rspdpipe.send("**ERROR %s: %s" % (str(exctype), str(excval)))
-            else:
-                self.__rspdpipe.send("**ERROR %s" % str(exctype))
+            try:
+                if excval:
+                    self.__rspdpipe.send("**ERROR %s: %s" % (str(exctype), str(excval)))
+                else:
+                    self.__rspdpipe.send("**ERROR %s" % str(exctype))
+            except Exception:
+                pass
             self.exitViewer()
 
     def processCommand(self, cmnd):
@@ -666,7 +714,7 @@ class PipedImagerPQ(QMainWindow):
         try:
             cmndact = cmnd["action"]
         except KeyError:
-            raise ValueError("Unknown command %s" % str(cmnd))
+            raise ValueError("Unknown command '%s'" % str(cmnd))
 
         if cmndact == "clear":
             try:
@@ -677,7 +725,7 @@ class PipedImagerPQ(QMainWindow):
         elif cmndact == "exit":
             self.exitViewer()
         elif cmndact == "hide":
-            self.hide()
+            self.showMinimized()
         elif cmndact == "screenInfo":
             scrnrect = QApplication.desktop().availableGeometry()
             info = ( self.physicalDpiX(), self.physicalDpiY(),
@@ -690,10 +738,7 @@ class PipedImagerPQ(QMainWindow):
                 bkgcolor = None
             self.redrawScene(bkgcolor)
         elif cmndact == "rescale":
-            newscale = float(cmnd["factor"])
-            if newscale <= 0.0:
-                raise ValueError("invalid scaling factor")
-            self.scaleScene(newscale, True)
+            self.scaleScene(float(cmnd["factor"]), True)
         elif cmndact == "resize":
             mysize = self.__helper.getSizeFromCmnd(cmnd)
             self.resizeScene(mysize.width(), mysize.height())
@@ -718,8 +763,8 @@ class PipedImagerPQ(QMainWindow):
             if myvalue:
                 self.__lastformat = myvalue.lower()
         elif cmndact == "show":
-            if self.isHidden():
-                self.showNormal()
+            if not self.isVisible():
+                self.show()
         elif cmndact == "noalpha":
             # ignore any alpha channel values in colors
             self.__noalpha = True
@@ -787,18 +832,18 @@ class _CommandSubmitterPQ(QDialog):
             cmndstr = str(self.__cmndlist[self.__nextcmnd])
             if len(cmndstr) > 188:
                 cmndstr = cmndstr[:188] + '...'
-            print "Command: %s" % cmndstr
+            print("Command: %s" % cmndstr)
             self.__cmndpipe.send(self.__cmndlist[self.__nextcmnd])
             self.__nextcmnd += 1
             while self.__rspdpipe.poll():
-                print "Response: %s" % str(self.__rspdpipe.recv())
+                print("Response: %s" % str(self.__rspdpipe.recv()))
         except IndexError:
             self.__rspdpipe.close()
             self.__cmndpipe.close()
             self.close()
 
 
-if __name__ == "__main__":
+def _test_pipedimagerpq():
     # vertices of a pentagon (roughly) centered in a 1000 x 1000 square
     pentagonpts = ( (504.5, 100.0), (100.0, 393.9),
                     (254.5, 869.4), (754.5, 869.4),
@@ -845,8 +890,8 @@ if __name__ == "__main__":
     # not a good way to get the pixel data
     testimgdata = bytearray(testimgheight * testimgstride)
     k = 0
-    for pty in xrange(testimgheight):
-        for ptx in xrange(testimgwidth):
+    for pty in range(testimgheight):
+        for ptx in range(testimgwidth):
             pixval = testimage.pixel(ptx, pty)
             (aval, rgbval) = divmod(pixval, 256 * 256 * 256)
             (rval, gbval) = divmod(rgbval, 256 * 256)
@@ -865,7 +910,7 @@ if __name__ == "__main__":
                         "width":testimgwidth,
                         "height":testimgheight,
                         "stride":testimgstride } )
-    for k in xrange(testnumblocks):
+    for k in range(testnumblocks):
         if k < (testnumblocks - 1):
             blkdata = testimgdata[k*testblocksize:(k+1)*testblocksize]
         else:
@@ -891,3 +936,6 @@ if __name__ == "__main__":
     if testresult != 0:
         sys.exit(testresult)
 
+if __name__ == "__main__":
+    _test_pipedimagerpq()
+
