@@ -13,34 +13,57 @@ Project (TMAP) of the National Oceanographic and Atmospheric
 Administration's (NOAA) Pacific Marine Environmental Lab (PMEL).
 '''
 
+from __future__ import print_function
+
+import sys
+import os
+import signal
+import time
+import math
+
 import sip
 try:
     sip.setapi('QVariant', 2)
 except AttributeError:
     pass
 
-from PyQt4.QtCore import Qt, QPointF, QRect, QRectF, QSize, QSizeF, \
-                         QString, QTimer
-from PyQt4.QtGui  import QAction, QApplication, QBrush, QColor, QDialog, \
-                         QFileDialog, QFontMetricsF, QImage, QLabel, \
-                         QMainWindow, QMessageBox, QPainter, QPalette, \
-                         QPen, QPicture, QPixmap, QPolygonF, QPrinter, \
-                         QPushButton, QScrollArea, QTextDocument
+if sys.version_info[0] > 2:
+    # First try to import PyQt5, then try PyQt4 if that fails
+    try:
+        import PyQt5
+        QT_VERSION = 5
+    except ImportError:
+        import PyQt4
+        QT_VERSION = 4
+else:
+    # PyQt5 requires Python3.x, so only try PyQt4
+    import PyQt4
+    QT_VERSION = 4
 
-try:
+# Now that the PyQt version is determined, import the parts
+# allowing any import errors to propagate out
+if QT_VERSION == 5:
+    from PyQt5.QtCore    import Qt, QPointF, QRect, QRectF, QSize, QSizeF, QTimer
+    from PyQt5.QtGui     import QBrush, QColor, QFontMetricsF, QImage, QPainter, \
+                                QPalette, QPen, QPicture, QPixmap, QPolygonF, \
+                                QTextDocument
+    from PyQt5.QtWidgets import QAction, QApplication, QDialog, QFileDialog, QLabel, \
+                                QMainWindow, QMessageBox, QPushButton, QScrollArea
+    from PyQt5.QtSvg     import QSvgGenerator
+    from PyQt5.QtPrintSupport import QPrinter
+else:
+    from PyQt4.QtCore import Qt, QPointF, QRect, QRectF, QSize, QSizeF, QTimer, QString
+    from PyQt4.QtGui  import QAction, QApplication, QBrush, QColor, QDialog, \
+                             QFileDialog, QFontMetricsF, QImage, QLabel, \
+                             QMainWindow, QMessageBox, QPainter, QPalette, \
+			     QPen, QPicture, QPixmap, QPolygonF, QPrinter, \
+			     QPushButton, QScrollArea, QTextDocument
     from PyQt4.QtSvg  import QSvgGenerator
-    HAS_QSvgGenerator = True
-except ImportError:
-    HAS_QSvgGenerator = False
 
-from cmndhelperpq import CmndHelperPQ
-from scaledialogpq import ScaleDialogPQ
 from multiprocessing import Pipe, Process
-import sys
-import time
-import os
-import signal
-import math
+
+from pipedviewer.cmndhelperpq import CmndHelperPQ
+from pipedviewer.scaledialogpq import ScaleDialogPQ
 
 
 class PipedViewerPQ(QMainWindow):
@@ -58,9 +81,7 @@ class PipedViewerPQ(QMainWindow):
           "outline":{"color":"black"},
           "location":(250,350) }
 
-    The command { "action":"exit" } will shutdown the viewer and is
-    the only way the viewer can be closed.  GUI actions can only hide
-    the viewer.
+    The command { "action":"exit" } will shutdown the viewer.
     '''
 
     def __init__(self, cmndpipe, rspdpipe):
@@ -151,9 +172,6 @@ class PipedViewerPQ(QMainWindow):
         self.__aboutqtact = QAction(self.tr("About &Qt"), self,
                                 statusTip=self.tr("Show information about the Qt library"),
                                 triggered=self.aboutQtMsg)
-        self.__exitact = QAction(self.tr("&Exit"), self,
-                                statusTip=self.tr("Shut down the viewer"),
-                                triggered=self.exitViewer)
         self.createMenus()
         # Set the initial size of the viewer
         self.__framedelta = 4
@@ -178,13 +196,9 @@ class PipedViewerPQ(QMainWindow):
         sceneMenu.addAction(self.__scaleact)
         sceneMenu.addAction(self.__saveact)
         sceneMenu.addAction(self.__redrawact)
-        # sceneMenu.addSeparator()
-        # sceneMenu.addAction(self.__hideact)
         helpMenu = menuBar.addMenu(menuBar.tr("&Help"))
         helpMenu.addAction(self.__aboutact)
         helpMenu.addAction(self.__aboutqtact)
-        helpMenu.addSeparator()
-        helpMenu.addAction(self.__exitact)
 
     def showEvent(self, event):
         '''
@@ -212,21 +226,22 @@ class PipedViewerPQ(QMainWindow):
 
     def closeEvent(self, event):
         '''
-        Override so the viewer cannot be closed from the
-        user selecting the windowframe close ('X') button.
-        Instead only hide the window.
+        Override so will just minimize if not shutting down
         '''
-        if self.__shuttingdown:
+        # If shutting down or minimized (escape hatch), 
+        # go ahead and close/exit
+        if self.__shuttingdown or self.isMinimized():
+            self.__timer.stop()
             event.accept()
-        else:
-            event.ignore()
-            self.hide()
+            return
+        # Otherwise just minimize the window
+        event.ignore()
+        self.showMinimized()
 
     def exitViewer(self):
         '''
         Close and exit the viewer.
         '''
-        self.__timer.stop()
         self.__shuttingdown = True
         self.close()
 
@@ -241,11 +256,12 @@ class PipedViewerPQ(QMainWindow):
             "The controlling application, however, may be unaware " \
             "of these modifications made to the image. " \
             "\n\n" \
-            "Normally, the controlling program will exit the viewer " \
-            "when it is no longer needed.  The Help -> Exit menu item " \
-            "should not normally be used.  It is provided when problems " \
-            "occur and the controlling program cannot shut down the " \
-            "viewer properly. " \
+            "'Closing' this window (the window frame 'X' button) " \
+            "actually only minimizes this viewer as PyFerret expects " \
+            "it to exist until PyFerret closes it.  Selecting the " \
+            "close menu option from the *minimized* window (using " \
+            "a right mouse click) will actually close (exit) the " \
+            "viewer if PyFerret exited and left it up.  " \
             "\n\n" \
             "PipedViewerPQ was developed by the Thermal Modeling and Analysis " \
             "Project (TMAP) of the National Oceanographic and Atmospheric " \
@@ -287,11 +303,9 @@ class PipedViewerPQ(QMainWindow):
         QApplication.setOverrideCursor(Qt.WaitCursor)
         # create the incomplete status message
         if (first + 1) < len(self.__viewpics):
-            mymsg = self.tr("%s (piece %%1 of %%2)" % statusmsg)
-            endstr = str(len(self.__viewpics))
+            mymsg = "%s (piece %%s of %s)" % (statusmsg, str(len(self.__viewpics)))
         else:
-            mymsg = self.tr("%s (piece %%1)" % statusmsg)
-            endstr = None
+            mymsg = "%s (piece %%s)" % statusmsg
         # get the origin for drawing the pictures after scaling
         myorigin = QPointF(leftx, uppery)
         # set the scaling factor for the pictures
@@ -302,10 +316,7 @@ class PipedViewerPQ(QMainWindow):
         for (viewpic, _) in self.__viewpics[first:]:
             k += 1
             # show the progress message
-            if endstr != None:
-                self.statusBar().showMessage( mymsg.arg(str(k)).arg(endstr) )
-            else:
-                self.statusBar().showMessage( mymsg.arg(str(k)) )
+            self.statusBar().showMessage( self.tr(mymsg % str(k)) )
             # draw the picture
             painter.drawPicture(myorigin, viewpic)
             if returnregion:
@@ -338,7 +349,7 @@ class PipedViewerPQ(QMainWindow):
         viewer is visible.
         '''
         if not ignorevis:
-            if self.isHidden() or self.isMinimized():
+            if self.isMinimized() or not self.isVisible():
                 # Not shown, so do not waste time drawing
                 return
         if self.__createpixmap:
@@ -499,7 +510,7 @@ class PipedViewerPQ(QMainWindow):
         the window size is changed by the user and auto-scaling is turn on.
 
         Returns:
-            True if the scene was resized
+            True if scaling of this scene is done (no window resize)
             False if the a new resize command was issued
         '''
         barheights = self.menuBar().height() + self.statusBar().height()
@@ -539,45 +550,59 @@ class PipedViewerPQ(QMainWindow):
         already scaled, that scaling is "removed" before this scaling
         factor is applied.  If resizewin is True, the main window is 
         resized to accommodate this new scaled scene size.
+
+        If factor is zero, just switch to auto-scaling at the current
+        window size.  If factor is negative, rescale using the absolute
+        value (possibly resizing the window) then switch to auto-scaling.
         '''
-        newfactor = float(factor)
-        newlabwidth = int(newfactor * self.__scenewidth + 0.5)
-        newlabheight = int(newfactor * self.__sceneheight + 0.5)
-        if (newlabwidth < self.__minsize) or (newlabheight < self.__minsize):
-            # Set to minimum size
-            if self.__scenewidth <= self.__sceneheight:
-                newfactor = float(self.__minsize) / float(self.__scenewidth)
-            else:
-                newfactor = float(self.__minsize) / float(self.__sceneheight)
+        fltfactor = float(factor)
+        if fltfactor != 0.0:
+            if resizewin:
+                # from command - turn off autoscaling for the following
+                # then turn back on if appropriate
+                self.__autoscale = False
+            newfactor = abs(fltfactor)
             newlabwidth = int(newfactor * self.__scenewidth + 0.5)
             newlabheight = int(newfactor * self.__sceneheight + 0.5)
-        oldlabwidth = int(self.__scalefactor * self.__scenewidth + 0.5)
-        oldlabheight = int(self.__scalefactor * self.__sceneheight + 0.5)
-        if (newlabwidth != oldlabwidth) or (newlabheight != oldlabheight):
-            # Set the new scaling factor
-            self.__scalefactor = newfactor
-            # Resize the label and set label values
-            # so the scrollarea knows of the new size
-            self.__label.setMinimumSize(newlabwidth, newlabheight)
-            self.__label.resize(newlabwidth, newlabheight)
-            # mark that the pixmap needs to be recreated
-            self.__createpixmap = True
-            # Redraw the scene from the beginning
-            self.redrawScene()
-        if resizewin:
-            # resize the main window (if possible)
-            barheights = self.menuBar().height() + self.statusBar().height()
-            mwheight = newlabheight + barheights + self.__framedelta
-            mwwidth = newlabwidth + self.__framedelta
-            # Do not exceed the available real estate on the screen.
-            # If autoscaling is in effect, the resize will trigger 
-            # any required adjustments.
-            scrnrect = QApplication.desktop().availableGeometry()
-            if mwwidth > 0.95 * scrnrect.width():
-                mwwidth = int(0.9 * scrnrect.width())
-            if mwheight > 0.95 * scrnrect.height():
-                mwheight = int(0.9 * scrnrect.height())
-            self.resize(mwwidth, mwheight)
+            if (newlabwidth < self.__minsize) or (newlabheight < self.__minsize):
+                # Set to minimum size
+                if self.__scenewidth <= self.__sceneheight:
+                    newfactor = float(self.__minsize) / float(self.__scenewidth)
+                else:
+                    newfactor = float(self.__minsize) / float(self.__sceneheight)
+                newlabwidth = int(newfactor * self.__scenewidth + 0.5)
+                newlabheight = int(newfactor * self.__sceneheight + 0.5)
+            oldlabwidth = int(self.__scalefactor * self.__scenewidth + 0.5)
+            oldlabheight = int(self.__scalefactor * self.__sceneheight + 0.5)
+            if (newlabwidth != oldlabwidth) or (newlabheight != oldlabheight):
+                # Set the new scaling factor
+                self.__scalefactor = newfactor
+                # Resize the label and set label values
+                # so the scrollarea knows of the new size
+                self.__label.setMinimumSize(newlabwidth, newlabheight)
+                self.__label.resize(newlabwidth, newlabheight)
+                # mark that the pixmap needs to be recreated
+                self.__createpixmap = True
+                # Redraw the scene from the beginning
+                self.redrawScene()
+            if resizewin:
+                # resize the main window (if possible)
+                barheights = self.menuBar().height() + self.statusBar().height()
+                mwheight = newlabheight + barheights + self.__framedelta
+                mwwidth = newlabwidth + self.__framedelta
+                # Do not exceed the available real estate on the screen.
+                # If autoscaling is in effect, the resize will trigger 
+                # any required adjustments.
+                scrnrect = QApplication.desktop().availableGeometry()
+                if mwwidth > 0.95 * scrnrect.width():
+                    mwwidth = int(0.9 * scrnrect.width())
+                if mwheight > 0.95 * scrnrect.height():
+                    mwheight = int(0.9 * scrnrect.height())
+                self.resize(mwwidth, mwheight)
+        if fltfactor <= 0.0:
+            # From command - turn on autoscaling
+            self.__autoscale = True
+            self.autoScaleScene();
 
     def inquireSaveFilename(self):
         '''
@@ -594,6 +619,8 @@ class PipedViewerPQ(QMainWindow):
                           self.tr("PDF - Portable Document Format (*.pdf)") ),
                         ( "ps",
                           self.tr("PS - PostScript (*.ps)") ),
+                        ( "svg",
+                          self.tr("SVG - Scalable Vector Graphics (*.svg)") ),
                         ( "bmp",
                           self.tr("BMP - Windows Bitmap (*.bmp)") ),
                         ( "ppm",
@@ -602,35 +629,41 @@ class PipedViewerPQ(QMainWindow):
                           self.tr("XPM - X11 Pixmap (*.xpm)") ),
                         ( "xbm",
                           self.tr("XBM - X11 Bitmap (*.xbm)") ), ]
-        if HAS_QSvgGenerator:
-            formattypes.insert(5, ( "svg",
-                          self.tr("SVG - Scalable Vector Graphics (*.svg)") ) )
-        # tr returns QStrings so the following does not work
-        # filters = ";;".join( [ t[1] for t in formattypes ] )
-        filters = QString(formattypes[0][1])
-        for typePair in formattypes[1:]:
-            filters.append(";;")
-            filters.append(typePair[1])
-        # getSaveFileNameAndFilter does not want to accept a default filter
-        # for (fmt, fmtQName) in formattypes:
-        #     if self.__lastformat == fmt:
-        #         dfltfilter = fmtQName
-        #         break
-        # else:
-        #     dfltfilter = formattypes[0][1]
-        # getSaveFileNameAndFilter is a PyQt (but not Qt?) method
-        (fileName, fileFilter) = QFileDialog.getSaveFileNameAndFilter(self,
-                                      self.tr("Save the current image as "),
-                                      self.__lastfilename, filters)
-        if fileName:
+        if QT_VERSION == 5:
+            # tr returns Python strings in PyQt5/Python3
+            filters = ";;".join( [ t[1] for t in formattypes ] )
             for (fmt, fmtQName) in formattypes:
-                if fmtQName.compare(fileFilter) == 0:
-                    fileFormat = fmt
+                if self.__lastformat == fmt:
+                    dfltfilter = fmtQName
                     break
             else:
-                raise RuntimeError("Unexpected file format name '%s'" % fileFilter)
-            self.saveSceneToFile(fileName, fileFormat, None, 
-                                 None, None, None)
+                dfltfilter = formattypes[0][1]
+            (fileName, fileFilter) = QFileDialog.getSaveFileName(self,
+                self.tr("Save the current image as "), self.__lastfilename, filters, dfltfilter)
+            if fileName:
+                for (fmt, fmtQName) in formattypes:
+                    if fmtQName == fileFilter:
+                        fileFormat = fmt
+                        break
+                else:
+                    raise RuntimeError("Unexpected file format name '%s'" % fileFilter)
+        else:
+            # tr returns QStrings in PyQt4
+            filters = formattypes[0][1]
+            for typePair in formattypes[1:]:
+                filters.append(";;")
+                filters.append(typePair[1])
+            (fileName, fileFilter) = QFileDialog.getSaveFileNameAndFilter(self,
+                 self.tr("Save the current image as "), self.__lastfilename, filters)
+            if fileName:
+                for (fmt, fmtQName) in formattypes:
+                    if fmtQName.compare(fileFilter) == 0:
+                        fileFormat = fmt
+                        break
+                else:
+                    raise RuntimeError("Unexpected file format name '%s'" % fileFilter)
+        if fileName:
+            self.saveSceneToFile(fileName, fileFormat, None, None, None, None)
             self.__lastfilename = fileName
             self.__lastformat = fileFormat
 
@@ -672,6 +705,9 @@ class PipedViewerPQ(QMainWindow):
             if fileext == '.pdf':
                 # needs a PDF QPrinter
                 myformat = 'pdf'
+            elif fileext == '.eps':
+                # needs a PS QPrinter and never rotate
+                myformat = 'eps'
             elif fileext == '.ps':
                 # needs a PS QPrinter
                 myformat = 'ps'
@@ -701,10 +737,6 @@ class PipedViewerPQ(QMainWindow):
         else:
             myfilename = filename
 
-        # The RHEL5 distribution of Qt4 does not have a QSvgGenerator
-        if (not HAS_QSvgGenerator) and (myformat == 'svg'):
-            raise ValueError("Your version of Qt does not support generation of SVG files")
-
         if myannotations:
             annopicture = QPicture()
             annopainter = QPainter(annopicture)
@@ -719,14 +751,14 @@ class PipedViewerPQ(QMainWindow):
             annopicture = None
             annosize = None
 
-        if (myformat == 'ps') or (myformat == 'pdf'):
-            # Setup the QPrinter that will be used to create the PS or PDF file
+        if (myformat == 'ps') or (myformat == 'eps') or (myformat == 'pdf'):
+            # Setup the QPrinter that will be used to create the EPS, PS, or PDF file
             printer = QPrinter(QPrinter.HighResolution)
             printer.setOutputFileName(myfilename)
             # The print format is automatically set from the
             # filename extension; so the following is actually
             # only needed for absent or strange extensions
-            if myformat == 'ps':
+            if (myformat == 'ps') or (myformat == 'eps'):
                 printer.setOutputFormat(QPrinter.PostScriptFormat)
             else:
                 printer.setOutputFormat(QPrinter.PdfFormat)
@@ -749,7 +781,7 @@ class PipedViewerPQ(QMainWindow):
             # Set the image size
             try:
                 # Set custom paper size to just fit around the image
-                if ( imagewidth > imageheight ):
+                if (myformat != 'eps') and (imagewidth > imageheight):
                     printer.setPaperSize(QSizeF(imageheight, imagewidth), QPrinter.Inch)
                 else:
                     printer.setPaperSize(QSizeF(imagewidth, imageheight), QPrinter.Inch)
@@ -764,7 +796,7 @@ class PipedViewerPQ(QMainWindow):
             # No margins (setPageMargins introduced in 4.4)
             printer.setFullPage(True)
             # Default orientation
-            if ( imagewidth > imageheight ):
+            if (myformat != 'eps') and (imagewidth > imageheight):
                 printer.setOrientation(QPrinter.Landscape)
             else:
                 printer.setOrientation(QPrinter.Portrait)
@@ -810,7 +842,6 @@ class PipedViewerPQ(QMainWindow):
                                 widthscalefactor, "Saving", False)                
             painter.end()
         elif myformat == 'svg':
-            # if HAS_QSvgGenerator is False, it should never get here
             generator = QSvgGenerator()
             generator.setFileName(myfilename)
             if vectsize:
@@ -915,7 +946,8 @@ class PipedViewerPQ(QMainWindow):
                                 widthscalefactor, "Saving", False)                
             painter.end()
             # save the image to file
-            image.save(myfilename, myformat)
+            if not image.save(myfilename, myformat):
+                raise ValueError("Unable to save the plot as " + myfilename)
 
     def checkCommandPipe(self):
         '''
@@ -942,10 +974,13 @@ class PipedViewerPQ(QMainWindow):
             # EOFError should never arise from recv since
             # the call is after poll returns True
             (exctype, excval) = sys.exc_info()[:2]
-            if excval:
-                self.__rspdpipe.send("**ERROR %s: %s" % (str(exctype), str(excval)))
-            else:
-                self.__rspdpipe.send("**ERROR %s" % str(exctype))
+            try:
+                if excval:
+                    self.__rspdpipe.send("**ERROR %s: %s" % (str(exctype), str(excval)))
+                else:
+                    self.__rspdpipe.send("**ERROR %s" % str(exctype))
+            except Exception:
+                pass
             self.exitViewer()
 
     def processCommand(self, cmnd):
@@ -968,7 +1003,7 @@ class PipedViewerPQ(QMainWindow):
         elif cmndact == "exit":
             self.exitViewer()
         elif cmndact == "hide":
-            self.hide()
+            self.showMinimized()
         elif cmndact == "screenInfo":
             scrnrect = QApplication.desktop().availableGeometry()
             info = ( self.physicalDpiX(), self.physicalDpiY(),
@@ -986,8 +1021,6 @@ class PipedViewerPQ(QMainWindow):
             self.redrawScene(bkgcolor)
         elif cmndact == "rescale":
             newscale = float(cmnd["factor"])
-            if newscale <= 0.0:
-                raise ValueError("invalid scaling factor")
             self.scaleScene(newscale, True)
         elif cmndact == "resize":
             mysize = self.__helper.getSizeFromCmnd(cmnd)
@@ -1019,8 +1052,8 @@ class PipedViewerPQ(QMainWindow):
             if value:
                 self.__lastformat = value.lower()
         elif cmndact == "show":
-            if self.isHidden():
-                self.showNormal()
+            if not self.isVisible():
+                self.show()
         elif cmndact == "noalpha":
             self.__noalpha = True
         elif cmndact == "beginView":
@@ -1379,10 +1412,10 @@ class PipedViewerPQ(QMainWindow):
         sides = self.__helper.getSidesFromCmnd(cmnd)
         width = sides.right() - sides.left()
         if width <= 0.0:
-            raise ValueError("width of the rectangle in not positive")
+            raise ValueError("width of the rectangle (%s) in not positive" % str(width))
         height = sides.bottom() - sides.top()
         if height <= 0.0:
-            raise ValueError("height of the rectangle in not positive")
+            raise ValueError("height of the rectangle (%s) in not positive" % str(height))
         myrect = QRectF(sides.left(), sides.top(), width, height)
         self.__activepainter.setRenderHint(QPainter.Antialiasing,
                                            False)
@@ -1430,7 +1463,9 @@ class PipedViewerPQ(QMainWindow):
             myfont = self.__activepainter.font()
         myfontmetrics = QFontMetricsF(myfont)
         mytext = cmnd["text"]
-        width = myfontmetrics.width(QString.fromUtf8(mytext))
+        if QT_VERSION == 4:
+            mytext = QString.fromUtf8(mytext)
+        width = myfontmetrics.width(mytext)
         height = myfontmetrics.height()
         return (width, height)
 
@@ -1479,7 +1514,10 @@ class PipedViewerPQ(QMainWindow):
                 self.__activepainter.rotate(rotdeg)
             except KeyError:
                 pass
-            self.__activepainter.drawText(0, 0, QString.fromUtf8(mytext))
+            
+            if QT_VERSION == 4:
+                mytext = QString.fromUtf8(mytext)
+            self.__activepainter.drawText(0, 0, mytext)
             self.__drawcount += 1
         finally:
             # return the painter to the default state
@@ -1563,18 +1601,18 @@ class _CommandSubmitterPQ(QDialog):
         or shutdown if there are no more commands to submit.
         '''
         try:
-            print "Command: %s" % str(self.__cmndlist[self.__nextcmnd])
+            print("Command: %s" % str(self.__cmndlist[self.__nextcmnd]))
             self.__cmndpipe.send(self.__cmndlist[self.__nextcmnd])
             self.__nextcmnd += 1
             while self.__rspdpipe.poll():
-                print "Response: %s" % str(self.__rspdpipe.recv())
+                print("Response: %s" % str(self.__rspdpipe.recv()))
         except IndexError:
             self.__rspdpipe.close()
             self.__cmndpipe.close()
             self.close()
 
 
-if __name__ == "__main__":
+def _test_pipedviewerpq():
     # vertices of a pentagon (roughly) centered in a 1000 x 1000 square
     pentagonpts = ( (504.5, 100.0), (100.0, 393.9),
                     (254.5, 869.4), (754.5, 869.4),
@@ -1751,3 +1789,6 @@ if __name__ == "__main__":
     if result != 0:
         sys.exit(result)
 
+if __name__ == "__main__":
+    _test_pipedviewerpq()
+
