@@ -15,6 +15,10 @@ This package was developed by the Thermal Modeling and Analysis Project
 Pacific Marine Environmental Lab (PMEL).
 '''
 
+from __future__ import print_function
+
+import sys
+
 from pyferret.graphbind.abstractpyferretbindings import AbstractPyFerretBindings
 from pipedviewer import PipedViewer
 
@@ -34,6 +38,12 @@ class PyFerretBindings(AbstractPyFerretBindings):
         '''
         super(PyFerretBindings, self).__init__()
         self.__window = None
+        # short wait time in seconds for error responses 
+        # to high level (not drawing) commands
+        self.__shortwait = 0.01
+        # long wait time in seconds for error responses
+        # to major operations; eg, save
+        self.__longwait = 1.0
 
     def createPipedViewerWindow(self, viewertype, title, visible, noalpha):
         '''
@@ -62,7 +72,7 @@ class PyFerretBindings(AbstractPyFerretBindings):
             self.__window.submitCommand( {"action":"show"} )
         if noalpha:
             self.__window.submitCommand( {"action":"noalpha"} )
-        self.checkForErrorResponse()
+        self.checkForErrorResponse(self.__shortwait)
         return True
 
     def submitCommand(self, cmnd):
@@ -71,34 +81,45 @@ class PyFerretBindings(AbstractPyFerretBindings):
 
         Provided for use by functions defined in subclasses.
         '''
+        # Check for an error from any previous command
+        self.__checkForErrorResponse()
+        # If all okay, submit the command
         self.__window.submitCommand(cmnd)
 
     def checkForResponse(self, timeout=0.0):
         '''
-        Checks the response pipe for an object within the given
-        timeout number of in seconds.  If timeout is None, this
-        method will wait indefinitely for something to be given
-        on the responds pipe.  If nothing is obtained within the
-        given timeout, None is returned.
-
-        Provided for use by functions defined in subclasses.
+        Checks the response pipe for an object within the given timeout 
+        number of in seconds.  If timeout is None, this method will wait 
+        indefinitely for something to be given on the responds pipe.  If 
+        timeout is zero (the default) the method does not wait if there 
+        is no response waiting.  If nothing is obtained within the given 
+        timeout, None is returned.
         '''
         result = self.__window.checkForResponse(timeout)
         return result
 
-    def checkForErrorResponse(self):
+    def checkForErrorResponse(self, timeout=0.0):
         '''
         Checks the response pipe for a message.  If anything is found,
         a RuntimeError is raised with the string of the full response.
+        If timeout is zero (the default) the method does not wait if there
+        is no response waiting; otherwise the method waits the given number
+        of seconds for a response to arrive.  An IllegalArgumentException
+        if raised if timeout is None, as error responses should not be an
+        expected result (thus an indefinite wait makes no sense).
         '''
+        if timeout is None:
+            raise IllegalArgumentException("timeout to checkForErrorResponse is None")
         fullresponse = None
-        response = self.__window.checkForResponse()
+        # Only wait (if timeout not zero) on the first check for a response
+        response = self.__window.checkForResponse(timeout)
         while response:
             if fullresponse:
                 fullresponse += '\n'
                 fullresponse += str(response)
             else:
                 fullresponse = str(response)
+            # Any further messages associated with this error should be immediate
             response = self.__window.checkForResponse()
         if fullresponse:
             raise RuntimeError(fullresponse)
@@ -113,7 +134,8 @@ class PyFerretBindings(AbstractPyFerretBindings):
         '''
         try:
             self.__window.submitCommand( { "action":"exit" } )
-            self.checkForErrorResponse()
+            # wait briefly for any error messages
+            self.checkForErrorResponse(self.__shortwait)
             self.__window.waitForViewerExit()
         finally:
             self.__window = None
@@ -144,7 +166,6 @@ class PyFerretBindings(AbstractPyFerretBindings):
         if formatname:
             cmnd["format"] = formatname
         self.__window.submitCommand(cmnd)
-        self.checkForErrorResponse()
 
     def setAntialias(self, antialias):
         '''
@@ -154,7 +175,6 @@ class PyFerretBindings(AbstractPyFerretBindings):
         cmnd = { "action":"antialias",
                  "antialias":bool(antialias) }
         self.__window.submitCommand(cmnd)
-        self.checkForErrorResponse()
         
     def beginView(self, leftfrac, bottomfrac, rightfrac, topfrac, clipit):
         '''
@@ -189,7 +209,6 @@ class PyFerretBindings(AbstractPyFerretBindings):
                               "top":topfracflt, "bottom":bottomfracflt }, 
                  "clip":bool(clipit) }
         self.__window.submitCommand(cmnd)
-        self.checkForErrorResponse()
 
     def clipView(self, clipit):
         '''
@@ -201,14 +220,12 @@ class PyFerretBindings(AbstractPyFerretBindings):
         cmnd = { "action":"clipView",
                  "clip":bool(clipit) }
         self.__window.submitCommand(cmnd)
-        self.checkForErrorResponse()
 
     def endView(self):
         '''
         Close a View in the PipedViewer Window
         '''
         self.__window.submitCommand( { "action":"endView" } )
-        self.checkForErrorResponse()
 
     def beginSegment(self, segid):
         '''
@@ -221,7 +238,6 @@ class PyFerretBindings(AbstractPyFerretBindings):
         cmnd = { "action":"beginSegment",
                  "segid":segid }
         self.__window.submitCommand(cmnd)
-        self.checkForErrorResponse()
 
     def endSegment(self):
         '''
@@ -229,7 +245,6 @@ class PyFerretBindings(AbstractPyFerretBindings):
         '''
         cmnd = { "action":"endSegment" }
         self.__window.submitCommand(cmnd)
-        self.checkForErrorResponse()
 
     def deleteSegment(self, segid):
         '''
@@ -241,7 +256,6 @@ class PyFerretBindings(AbstractPyFerretBindings):
         cmnd = { "action":"deleteSegment",
                  "segid":segid }
         self.__window.submitCommand(cmnd)
-        self.checkForErrorResponse()
 
     def updateWindow(self):
         '''
@@ -249,7 +263,6 @@ class PyFerretBindings(AbstractPyFerretBindings):
         '''
         cmnd = { "action":"update" }
         self.__window.submitCommand(cmnd)
-        self.checkForErrorResponse()
 
     def clearWindow(self, bkgcolor):
         '''
@@ -266,7 +279,6 @@ class PyFerretBindings(AbstractPyFerretBindings):
             cmnd = { }
         cmnd["action"] = "clear"
         self.__window.submitCommand(cmnd)
-        self.checkForErrorResponse()
 
     def redrawWindow(self, bkgcolor):
         '''
@@ -284,7 +296,6 @@ class PyFerretBindings(AbstractPyFerretBindings):
             cmnd = { }
         cmnd["action"] = "redraw"
         self.__window.submitCommand(cmnd)
-        self.checkForErrorResponse()
 
     def resizeWindow(self, width, height):
         '''
@@ -300,7 +311,6 @@ class PyFerretBindings(AbstractPyFerretBindings):
                  "width":width,
                  "height":height }
         self.__window.submitCommand(cmnd)
-        self.checkForErrorResponse()
 
     def scaleWindow(self, scale):
         '''
@@ -312,7 +322,6 @@ class PyFerretBindings(AbstractPyFerretBindings):
         cmnd = { "action":"rescale",
                  "factor":scale }
         self.__window.submitCommand(cmnd)
-        self.checkForErrorResponse()
 
     def windowScreenInfo(self):
         '''
@@ -365,7 +374,6 @@ class PyFerretBindings(AbstractPyFerretBindings):
         else:
             cmnd = { "action":"hide" }
         self.__window.submitCommand(cmnd)
-        self.checkForErrorResponse()
 
     def saveWindow(self, filename, fileformat, transparent,
                    xinches, yinches, xpixels, ypixels, annotations):
@@ -406,7 +414,8 @@ class PyFerretBindings(AbstractPyFerretBindings):
         cmnd["rastsize"] = { "width":xpixels, "height":ypixels }
         cmnd["annotations"] = annotations
         self.__window.submitCommand(cmnd)
-        self.checkForErrorResponse()
+        # Wait awhile to see if there was any problems
+        self.checkForErrorResponse(self.__longwait)
 
     def createColor(self, redfrac, greenfrac, bluefrac, opaquefrac):
         '''
@@ -618,7 +627,6 @@ class PyFerretBindings(AbstractPyFerretBindings):
         cmnd = { "action":"setWidthFactor",
                  "factor":widthfactor }
         self.__window.submitCommand(cmnd)
-        self.checkForErrorResponse()
         
     def drawMultiline(self, ptsx, ptsy, pen):
         '''
@@ -639,7 +647,6 @@ class PyFerretBindings(AbstractPyFerretBindings):
                  "points":points,
                  "pen":pen }
         self.__window.submitCommand(cmnd)
-        self.checkForErrorResponse()
 
     def drawPoints(self, ptsx, ptsy, symbol, color, ptsize):
         '''
@@ -668,7 +675,6 @@ class PyFerretBindings(AbstractPyFerretBindings):
         cmnd["symbol"] = symbol
         cmnd["size"] = ptsize
         self.__window.submitCommand(cmnd)
-        self.checkForErrorResponse()
 
     def drawPolygon(self, ptsx, ptsy, brush, pen):
         '''
@@ -694,7 +700,6 @@ class PyFerretBindings(AbstractPyFerretBindings):
         if pen:
             cmnd["outline"] = pen
         self.__window.submitCommand(cmnd)
-        self.checkForErrorResponse()
 
     def drawRectangle(self, left, bottom, right, top, brush, pen):
         '''
@@ -721,7 +726,6 @@ class PyFerretBindings(AbstractPyFerretBindings):
         if pen:
             cmnd["outline"] = pen
         self.__window.submitCommand(cmnd)
-        self.checkForErrorResponse()
 
     def textSize(self, text, font):
         '''
@@ -792,7 +796,6 @@ class PyFerretBindings(AbstractPyFerretBindings):
         if rotate != 0.0:
             cmnd["rotate"] = rotate
         self.__window.submitCommand(cmnd)
-        self.checkForErrorResponse()
 
 
 class PViewerPQPyFerretBindings(PyFerretBindings):
@@ -800,7 +803,7 @@ class PViewerPQPyFerretBindings(PyFerretBindings):
     PyFerretBindings using a PipedViewerPQ as the viewer.
     '''
 
-    def createWindow(self, title, visible, noalpha):
+    def createWindow(self, title, visible, noalpha, rasteronly):
         '''
         Creates PyFerret bindings using a PipedViewerPQ.
 
@@ -808,6 +811,7 @@ class PViewerPQPyFerretBindings(PyFerretBindings):
             title: display title for the Window
             visible: display Window on start-up?
             noalpha: do not use the alpha channel in colors?
+            rasteronly: ignored
 
         Raises a RuntimeError if an active window is already associated
         with these bindings, or if there were problems with creating
@@ -835,7 +839,7 @@ class PImagerPQPyFerretBindings(PyFerretBindings):
     for the new scene to be displayed.
     '''
 
-    def createWindow(self, title, visible, noalpha):
+    def createWindow(self, title, visible, noalpha, rasteronly):
         '''
         Creates PyFerret bindings using a PipedImagerPQ.
 
@@ -843,6 +847,7 @@ class PImagerPQPyFerretBindings(PyFerretBindings):
             title: display title for the Window
             visible: display Window on start-up?
             noalpha: do not use the alpha channel in colors?
+            rasteronly: ignored
 
         Raises a RuntimeError if an active window is already associated
         with these bindings, or if there were problems with creating
@@ -876,10 +881,9 @@ class PImagerPQPyFerretBindings(PyFerretBindings):
                  "height":height,
                  "stride":stride }
         self.submitCommand(cmnd)
-        self.checkForErrorResponse()
         blocksize = 8192
         numblocks = (lenimgdata + blocksize - 1) // blocksize
-        for k in xrange(numblocks):
+        for k in range(numblocks):
             if k < (numblocks - 1):
                 blkdata = imagedata[k*blocksize:(k+1)*blocksize]
             else:
@@ -890,14 +894,15 @@ class PImagerPQPyFerretBindings(PyFerretBindings):
                      "startindex":k*blocksize,
                      "blockdata":blkdata }
             self.submitCommand(cmnd)
-            self.checkForErrorResponse()
+        # wait briefly at the end for any error messages
+        self.checkForErrorResponse(self.__shortwait)
 
 
 #
 # The following is for testing this module
 #
 
-if __name__ == "__main__":
+def _test_pyferretbindings():
     import pyferret
     import pyferret.graphbind
 
@@ -931,10 +936,10 @@ if __name__ == "__main__":
     # Initiate pyferret, but stay in python
     pyferret.init(None, False)
     for viewertype in ( "PipedViewerPQ", ):
-        print "Testing bindings for %s" % viewertype
+        print("Testing bindings for %s" % viewertype)
         # Create a viewer window
         title = viewertype + "Tester"
-        bindinst = pyferret.graphbind.createWindow(viewertype, title, True, False)
+        bindinst = pyferret.graphbind.createWindow(viewertype, title, True, False, False)
         # Resize the window to 500 x 500 pixels
         bindinst.resizeWindow(500, 500)
         # Turn on anti-aliasing
@@ -967,7 +972,10 @@ if __name__ == "__main__":
         bindinst.endView()
         # Window should already be shown, but just to make sure
         bindinst.showWindow(True)
-        raw_input("Press Enter to continue")
+        if sys.version_info[0] > 2:
+            input("Press Enter to continue")
+        else:
+            raw_input("Press Enter to continue")
         # Create a view of the whole window
         bindinst.beginView(0.0, 1.0, 1.0, 0.0, True)
         # Draw magenta points using various symbols
@@ -1010,7 +1018,10 @@ if __name__ == "__main__":
         bindinst.endView()
         # Window should already be shown, but just to make sure
         bindinst.showWindow(True)
-        raw_input("Press Enter to continue")
+        if sys.version_info[0] > 2:
+            input("Press Enter to continue")
+        else:
+            raw_input("Press Enter to continue")
         try:
             while 1:
                 bindinst.deleteColor(mycolors.pop())
@@ -1018,5 +1029,8 @@ if __name__ == "__main__":
             pass
         bindinst.deleteFont(myfont)
         bindinst.deleteWindow()
-        print "Done with bindings for %s" % viewertype
+        print("Done with bindings for %s" % viewertype)
+
+if __name__ == "__main__":
+    _test_pyferretbindings()
 
