@@ -1,5 +1,5 @@
 """
-A Python module for running Ferret.  
+A Python module for running Ferret.
 For running the Ferret engine that is part of this module:
     init or start must first be called to initialize Ferret
     resize can be used to resize Ferret's allocated memory block
@@ -20,16 +20,18 @@ For writing Ferret external functions in Python, see the help message
 for pyferret.pyefmethods, whose methods are imported into the pyferret
 module.
 
-The FerRegion, FerAxis, FerGrid, FerVar, FerPyVar, and FerDSet objects 
-assist in working with the Ferret engine in a Python environment, 
+The FerRegion, FerAxis, FerGrid, FerVar, FerPyVar, and FerDSet objects
+assist in working with the Ferret engine in a Python environment,
 reducing the need to know the Ferret language and syntax.
 
 The convenience methods for executing common Ferret commands are found
 under pyferret.fermethods and are imported into the pyferret module.
-These methods also help reduce the need to know the Ferret language 
+These methods also help reduce the need to know the Ferret language
 and syntax.
 """
 
+from __future__ import print_function
+
 import sys
 import os
 import atexit
@@ -39,52 +41,54 @@ except ImportError:
     pass
 import readline
 
-import libpyferret
-# also import everything (not starting with an underscore) from libpyferret 
+from pyferret import libpyferret
+# also import everything (not starting with an underscore) from libpyferret
 # so constants in that module are seen as part of this module
-from libpyferret import *
+from pyferret.libpyferret import *
 
 # methods for transferring data between the Ferret engine and Python
-import datamethods
+from pyferret import datamethods
 # also import the methods given in datamethods into pyferret
-from datamethods import *
+from pyferret.datamethods import *
 
 # methods to assist in writing Ferret external functions written in Python
-import pyefmethods
+from pyferret import pyefmethods
 # also import the methods given in pyefmethods into pyferret
-from pyefmethods import *
+from pyferret.pyefmethods import *
 
-# the FerRegion, FerAxis, FerGrid, FerVar, FerPyVar, and FerDSet objects 
-# for working with Ferret from Python
-import ferregion
-from ferregion import *
-import feraxis
-from feraxis import *
-import fergrid
-from fergrid import *
-import fervar
-from fervar import *
-import ferpyvar
-from ferpyvar import *
-import ferdset
-from ferdset import *
-import feraggdset
-from feraggdset import *
+# the Fer... objects for working with Ferret from Python
+from pyferret import ferregion
+from pyferret.ferregion import *
+from pyferret import feraxis
+from pyferret.feraxis import *
+from pyferret import fergrid
+from pyferret.fergrid import *
+from pyferret import fervar
+from pyferret.fervar import *
+from pyferret import ferpyvar
+from pyferret.ferpyvar import *
+from pyferret import ferdset
+from pyferret.ferdset import *
+from pyferret import feraggdset
+from pyferret.feraggdset import *
+from pyferret import ferfmrcdset
+from pyferret.ferfmrcdset import *
+from pyferret import ferfmrcvar
+from pyferret.ferfmrcvar import *
 
 # Create "the" anonymous dataset
-anondset = FerDSet(qual=ferdset._anonymous_dataset_qualifier)
+anondset = FerDSet('', qual=ferdset._anonymous_dataset_qualifier)
 
 # convenience methods for executing common Ferret commands
-import fermethods
-from fermethods import *
+from pyferret import fermethods
+from pyferret.fermethods import *
 
 # bindings for the PyQt-based graphics engines
 import pipedviewer.pyferretbindings
 
-# the following should be in this (pyferret) directory, which should be examined first
-import filenamecompleter
-import graphbind
-import regrid
+from pyferret import filenamecompleter
+from pyferret import graphbind
+from pyferret import regrid
 
 def init(arglist=None, enterferret=True):
     """
@@ -109,10 +113,9 @@ def init(arglist=None, enterferret=True):
     ferret_help_message = \
     """
 
-    Usage:  pyferret  [-memsize <N>]  [-nodisplay]  [-nojnl]  [-noverify]
-                      [-secure]  [-server]  [-python]  [-version]  [-help] 
-                      [-quiet]  [-linebuffer]  [-batch [<filename>]]  
-                      [-transparent]  [-script <scriptname> [ <scriptarg> ... ]]
+    Usage:  pyferret  [-memsize <N>]  [-nodisplay]  [-png]  [-nojnl]  [-noverify]
+                      [-secure]  [-server]  [-python]  [-version]  [-help]
+                      [-quiet]  [-linebuffer]  [-script <scriptname> [ <scriptarg> ... ]]
 
        -memsize:     initialize the memory cache size to <N> (default 25)
                      mega (10^6) floats (where 1 float = 8 bytes)
@@ -120,9 +123,12 @@ def init(arglist=None, enterferret=True):
        -nodisplay    do not display to the console; a drawing can be saved
                      using the FRAME command in any of the supported file
                      formats.  The /QUALITY option of SET WINDOW will be
-                     ignored when this is specified.  The deprecated
-                     command-line options -unmapped and -gif are now
-                     aliases of this option.
+                     ignored when this is specified.
+
+       -png          do not display to the console and only raster (PNG)
+                     images can be saved.  Implies -nodisplay and removes
+                     support for deletion of parts (segments) of a plot;
+                     however, plots will be generated faster.
 
        -nojnl:       on startup do not open a journal file (can be turned
                      on later with SET MODE JOURNAL)
@@ -133,10 +139,10 @@ def init(arglist=None, enterferret=True):
        -secure:      restrict Ferret's capabilities (e.g., SPAWN and
                      EXIT /TOPYTHON are not permitted)
 
-       -server:      run Ferret in server mode (don't stop on message commands)
+       -server:      run in server mode (don't stop on message commands)
 
        -python:      start at the Python prompt instead of the Ferret prompt.
-                     The ferret prompt can be obtained using 'pyferret.run()'
+                     The Ferret prompt can be obtained using 'pyferret.run()'
 
        -version:     print the Ferret header with version number and quit
 
@@ -144,37 +150,43 @@ def init(arglist=None, enterferret=True):
 
        -quiet        do not display the startup header
 
-       -linebuffer   print each line of output or error messages as soon as 
-                     a full line is written.  Useful when redirecting these
-                     messages to a file.
-                     Note: 
-                       the enviroment variable GFORTRAN_UNBUFFERED_PRECONNECTED 
-                       needs to be set to 1 in order to unbuffer the Fortran 
+       -linebuffer   print each line of output or error messages as soon as
+                     a full line is written.  Useful when redirecting both
+                     types of messages to a single file.
+                     Note:
+                       the environment variable GFORTRAN_UNBUFFERED_PRECONNECTED
+                       needs to be set to 1 in order to unbuffer the Fortran
                        units for output and error messages
 
-       -batch:       draw to <filename> (default "ferret.png") instead of
-                     displaying to the console.  The file format will be
-                     guessed from the filename extension.  When using this
-                     option, new windows should not be created and the
-                     FRAME command should not be used.
-
-                     Use of -batch (and -transparent) is not recommended.
-                     Instead use the -nodisplay option and the FRAME
-                     /FILE=<filename> [ /TRANSPARENT ] command.
-
-       -transparent: use a transparent background instead of opaque white
-                     when saving to the file given by -batch
-
-       -script:      execute the script <scriptname> with any arguments 
-                     specified and exit (THIS MUST BE SPECIFIED LAST).  
-                     The -script option also implies the -nojnl, -noverify, 
+       -script:      execute the script <scriptname> with any arguments
+                     specified and exit. (THIS MUST BE SPECIFIED LAST.)
+                     The -script option also implies the -nojnl, -noverify,
                      -server, and -quiet options.
 
+       Deprecated command-line options:
+
+           -gif:         this option is now an alias for -png
+
+	   -unmapped:    this option is now an alias for -nodisplay
+
+           -batch:       draw to <filename> (default "ferret.png") instead of
+                         displaying to the console.  The file format will be
+                         guessed from the filename extension.  When using this
+                         option, new windows should not be created and the
+                         FRAME command should not be used.
+
+           -transparent: use a transparent background instead of opaque white
+                         when saving to the file given by -batch
+
+        Use of -batch and -transparent options is strongly discouraged.  Instead,
+        use the -nodisplay option and the FRAME /FILE=<filename> [ /TRANSPARENT ]
+        Ferret command.
     """
 
     my_metaname = None
     my_transparent = False
     my_unmapped = False
+    my_pngonly = False
     my_memsize = 25
     my_journal = True
     my_verify = True
@@ -216,11 +228,14 @@ def init(arglist=None, enterferret=True):
                     my_transparent = True
                 elif opt == "-nodisplay":
                     my_unmapped = True
-                elif opt == "-gif":
-                    my_unmapped = True
-                    my_metaname = "ferret.png"
                 elif opt == "-unmapped":
                     my_unmapped = True
+                elif opt == "-png":
+                    my_unmapped = True
+                    my_pngonly = True
+                elif opt == "-gif":
+                    my_unmapped = True
+                    my_pngonly = True
                 elif opt == "-nojnl":
                     my_journal = False
                 elif opt == "-noverify":
@@ -257,13 +272,13 @@ def init(arglist=None, enterferret=True):
                 else:
                     raise ValueError("unrecognized option '%s'" % opt)
                 k += 1
-        except ValueError, errmsg:
+        except ValueError as errmsg:
             # print the error message, then print the help message
-            print >>sys.stderr, "\n%s" % errmsg
+            print("\n%s" % errmsg, file=sys.stderr)
             print_help = True
         if print_help:
             # print the help message, then mark for exiting
-            print >>sys.stderr, ferret_help_message
+            print(ferret_help_message, file=sys.stderr)
             just_exit = True
         if just_exit:
             # print the ferret header then exit completely
@@ -477,8 +492,8 @@ def init(arglist=None, enterferret=True):
     # start ferret without journaling
     start(memsize=my_memsize, journal=False, verify=my_verify,
           restrict=my_restrict, server=my_server, metaname=my_metaname,
-          transparent=my_transparent, unmapped=my_unmapped, 
-          quiet=my_quiet, linebuffer=my_linebuffer)
+          transparent=my_transparent, unmapped=my_unmapped,
+          pngonly=my_pngonly, quiet=my_quiet, linebuffer=my_linebuffer)
 
     # define all the Ferret standard Python external functions
     for fname in std_pyefs:
@@ -493,7 +508,7 @@ def init(arglist=None, enterferret=True):
                 try:
                     result = run('go "%s"; exit /topy' % init_script)
                 except:
-                    print >>sys.stderr, " **Error: exception raised in runnning script %s" % init_script
+                    print(" **Error: exception raised in runnning script %s" % init_script, file=sys.stderr)
                     result = run('exit /program')
                     # should not get here
                     raise SystemExit
@@ -505,7 +520,7 @@ def init(arglist=None, enterferret=True):
         try:
             result = run('go %s; exit /program' % script_line)
         except:
-            print >>sys.stderr, " **Error: exception raised in running script %s" % script_line
+            print(" **Error: exception raised in running script %s" % script_line, file=sys.stderr)
         # If exception or if returned early, force shutdown
         result = run('exit /program')
         # should not get here
@@ -526,19 +541,23 @@ def init(arglist=None, enterferret=True):
 
 def start(memsize=25, journal=True, verify=False, restrict=False,
           server=False, metaname=None, transparent=False,
-          unmapped=False, quiet=False, linebuffer=False):
+          unmapped=False, pngonly=False, quiet=False, linebuffer=False):
     """
     Initializes Ferret.  This allocates the initial amount of memory
     for Ferret (from Python-managed memory), opens the journal file,
-    if requested, and sets Ferret's verify mode.  If restrict is True,
-    some Ferret commands will not be available (to provide a secured
-    session).  Once restrict is set, it cannot be unset.  If server
-    is True, Ferret will be run in server mode.  If metaname is not
-    empty this value is used as the initial filename for automatic
-    output of graphics, and the graphics viewer will not be displayed.
+    if requested, and sets Ferret's verify mode.
+    If restrict is True, some Ferret commands will not be available
+    (to provide a secured session).  Once restrict is set, it cannot
+    be unset.
+    If metaname is not empty this value is used as the initial filename
+    for automatic output of graphics, and the graphics viewer will not
+    be displayed.
+    If server is True, Ferret will be run in server mode.
     If unmapped is True, the graphics viewer will not be displayed.
+    If pngonly is True, only raster images can be saved and the
+    graphics viewer will not be displayed.
     If quiet is True, the Ferret start-up header is not displayed.
-    If linebuffer is True, stdout and stderr are set user line 
+    If linebuffer is True, stdout and stderr are set user line
     buffering.  This cannot be reset once set.
     This routine does NOT run any user initialization scripts.
 
@@ -553,13 +572,14 @@ def start(memsize=25, journal=True, verify=False, restrict=False,
         transparent: autosave (e.g., on exit) image files with a
                      transparent background?
         unmapped:    hide the graphics viewer?
+        pngonly:     only create raster (PNG) images?
         quiet:       do not display the Ferret start-up header?
-        linebuffer:  print each line of output or error messages as soon as 
+        linebuffer:  print each line of output or error messages as soon as
                      a full line is written?  Useful when redirecting these
                      messages to a file.
-                     Note: 
-                       the enviroment variable GFORTRAN_UNBUFFERED_PRECONNECTED 
-                       needs to be set to 1 in order to unbuffer the Fortran 
+                     Note:
+                       the enviroment variable GFORTRAN_UNBUFFERED_PRECONNECTED
+                       needs to be set to 1 in order to unbuffer the Fortran
                        units for output and error messages
     Returns:
         True is successful
@@ -600,8 +620,8 @@ def start(memsize=25, journal=True, verify=False, restrict=False,
     # the actual call to ferret's start
     success = libpyferret._start(flt_memsize, bool(journal), bool(verify),
                                  bool(restrict), bool(server), str_metaname,
-                                 bool(transparent), bool(unmapped), 
-                                 bool(quiet), bool(linebuffer))
+                                 bool(transparent), bool(unmapped),
+                                 bool(pngonly), bool(quiet), bool(linebuffer))
     if success:
         # register the libpyferret._quit function with atexit to ensure
         # open viewer windows do not hang a Python shutdown
@@ -613,9 +633,14 @@ def start(memsize=25, journal=True, verify=False, restrict=False,
         # Execute the $PYTHONSTARTUP file, if it exists and -secure not given
         if not restrict:
             try:
-                execfile(os.getenv('PYTHONSTARTUP', ''));
-            except IOError:
-                pass;
+                startfilename = os.getenv('PYTHONSTARTUP', '')
+                if startfilename:
+                    if sys.version_info[0] > 2:
+                        exec(compile(open(startfilename).read(), startfilename, 'exec'));
+                    else:
+                        execfile(startfilename)
+            except Exception:
+                pass
 
     return success
 
@@ -641,8 +666,8 @@ def resize(memsize):
             raise ValueError
     except:
         raise ValueError("memsize must be a positive number")
-    # the actual call
-    return libpyferret._resize(flt_memsize)
+    # the actual call - now just a ferret command handled internally
+    return libpyferret._run("SET MEMORY /SIZE=%d" % flt_memsize)
 
 
 def run(command=None):
@@ -727,14 +752,20 @@ def _readline(myprompt):
     Arguments:
         myprompt - prompt string to use
     Returns:
-        the string read in, or None if EOFError occurs
+        the string read in, or None if an Exception occurs
     """
     try:
-        if myprompt:
-            myline = raw_input(myprompt)
+        if sys.version_info[0] > 2:
+            if myprompt:
+                myline = input(myprompt)
+            else:
+                myline = input()
         else:
-            myline = raw_input()
-    except EOFError:
+            if myprompt:
+                myline = raw_input(myprompt)
+            else:
+                myline = raw_input()
+    except Exception:
         myline = None
 
     return myline
